/*
 * File:   Fast Hermite order 3
 * Author: Fabiola Pacheco
 * 
 * Created on 02/28/2017
 * Last modification on 07/31/2017
 */
#ifndef __itkHermiteImageFilter_h
#define __itkHermiteImageFilter_h
 
#include "itkImageToImageFilter.h"
 
namespace itk
{
  template< class TInputImage, class TOutputImage>
  class HermiteImageFilter : public ImageToImageFilter< TInputImage, TOutputImage >
  {
  public:
    /** Standard class typedefs. */
    typedef HermiteImageFilter Self;
    typedef ImageToImageFilter< TInputImage, TOutputImage > Superclass;
    typedef SmartPointer< Self > Pointer;
    
    typedef TInputImage InputImageType;
    typedef TOutputImage OutputImageType;
   
    /** Method for creation through the object factory. */
    itkNewMacro(Self);
   
    /** Run-time type information (and related methods). */
    itkTypeMacro(HermiteImageFilter, ImageToImageFilter);
   
    /**Coeffieciens of Hermite Transform
    *\GetCoef(0) = Coeffiecient 00
    *\GetCoef(1) = Coeffiecient 10
    *\GetCoef(2) = Coeffiecient 01
    *\GetCoef(3) = Coeffiecient 20
    *\GetCoef(4) = Coeffiecient 11
    *\GetCoef(5) = Coeffiecient 02
    *\GetCoef(6) = Coeffiecient 30
    *\GetCoef(7) = Coeffiecient 21
    *\GetCoef(8) = Coeffiecient 12
    *\GetCoef(9) = Coeffiecient 03
    */
    OutputImageType* GetCoef(int n);

    /**Rotated Coeffieciens of Hermite Transform
    *\GetRotCoef(1) = Rotation of Coeffiecient 10
    *\GetRotCoef(2) = Rotation of Coeffiecient 20
    *\GetRotCoef(3) = Rotation of Coeffiecient 30
    */
    OutputImageType* GetRotCoef(int n);

    /**Angle in radians*/
    OutputImageType* GetAngle();
   
  protected:
    HermiteImageFilter();
    ~HermiteImageFilter(){}
   
    /** Does the real work. */
    virtual void GenerateData();
   
    /**  Create the Output */
    DataObject::Pointer MakeOutput();
   
  private:
    HermiteImageFilter(const Self &); //purposely not implemented
    void operator=(const Self &);  //purposely not implemented

    OutputImageType* GetNthOutput(int n);
   
  };
} //namespace ITK 
 
#ifndef ITK_MANUAL_INSTANTIATION
#include "itkHermiteImageFilter.txx"
#endif 
 
#endif // __itkHermiteImageFilter_h