/*
 * File:   Fast Hermite order 3
 * Author: Fabiola Pacheco
 * 
 * Created on 02/28/2017
 * Last modification on 07/31/2017
 */
#ifndef __itkHermiteImageFilter_txx
#define __itkHermiteImageFilter_txx

#include "itkHermiteImageFilter.h"
 
#include "itkObjectFactory.h"
//#include <itkRescaleIntensityImageFilter.h>
 
namespace itk
{
 
  template< class TInputImage, class TOutputImage>
  HermiteImageFilter<TInputImage, TOutputImage>::HermiteImageFilter()
  {
    this->SetNumberOfRequiredOutputs(14);
    this->SetNumberOfRequiredInputs(1);
   
    for(int i=0;i<14;i++)
    this->SetNthOutput( i, this->MakeOutput() );
  }
   
  template< class TInputImage, class TOutputImage>
  void HermiteImageFilter<TInputImage, TOutputImage>::GenerateData()
  {
    OutputImageType::SizeType size = 
      this->GetInput()->GetLargestPossibleRegion().GetSize();
    
    OutputImageType::IndexType regionStart;
      regionStart[0] = 0; 
      regionStart[1] = 0; 
      
    OutputImageType::SizeType regionSize;
      regionSize[0] = size[0]; 
      regionSize[1] = size[1]; 
      
    OutputImageType::RegionType region;
      region.SetSize( regionSize );
      region.SetIndex( regionStart );
      
    for(int i=0;i<14;i++){
      OutputImageType::Pointer temp = this->GetNthOutput(i);
      temp->SetRegions(region);
      temp->Allocate();
    }
        
    itk::Vector<float, 18> L;
    itk::Vector<float, 11> MC;
    
    for(int i=0;i<11;i++){
      L[i] = 0.0;
      MC[i] = 0.0;
    }
    
    for(int i=11;i<18;i++){
      L[i] = 0.0;
    }
    
    itk::Image<float ,2>::Pointer MR = itk::Image<float ,2>::New();
    
    itk::Image<float ,2>::IndexType region2Start;
      region2Start[0] = 0; 
      region2Start[1] = 0; 
      
    itk::Image<float ,2>::SizeType region2Size;
      region2Size[0] = 7; 
      region2Size[1] = size[1]+5; 
      
    itk::Image<float ,2>::RegionType region2;
      region2.SetSize( region2Size );
      region2.SetIndex( region2Start );
      
    MR->SetRegions(region2);
    MR->Allocate();
    MR->FillBuffer(0.0);
    
    itk::Image<float ,3>::Pointer coef = itk::Image<float ,3>::New();
    
    itk::Image<float ,3>::IndexType region3Start;
      region3Start[0] = 0; 
      region3Start[1] = 0; 
      region3Start[2] = 0; 
      
    itk::Image<float ,3>::SizeType region3Size;
      region3Size[0] = size[0]+5; 
      region3Size[1] = size[1]+5; 
      region3Size[2] = 10; 
      
    itk::Image<float ,3>::RegionType region3;
      region3.SetSize( region3Size );
      region3.SetIndex( region3Start );
      
    coef->SetRegions(region3);
    coef->Allocate();
    coef->FillBuffer(0.0);
  
    itk::Image<unsigned char, 2>::IndexType pixelIndex;
    itk::Image<unsigned char, 2>::IndexType pixInd;
    itk::Image<unsigned char, 3>::IndexType pixC;
    
    itk::Image<float , 2>::IndexType MRindex;
    
    //calculate coefficients in each pixel
    for(int i=0;i<size[0]+5;i++){
      for(int j=0;j<size[1]+5;j++){
        pixelIndex[0] =i-2; 
        pixelIndex[1] =j-2;
        pixC[0] =i; 
        pixC[1] =j;
        MRindex[1] =j;  

        //pixelValue, zero-padding = contour of image
        if(pixelIndex[0]<0){
          if(pixelIndex[1]<0){
            pixInd[0]=0;
            pixInd[1]=0;
            L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
          }else{
            if(pixelIndex[1]>(size[1]-1)){
              pixInd[0]=0;
              pixInd[1]=size[1]-1;
              L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
            }else{//pixelIndex[1] in range
              pixInd[0]=0;
              pixInd[1]=pixelIndex[1];
              L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
            }
          }
        }else{
          if(pixelIndex[0]>(size[0]-1)){
            if(pixelIndex[1]<0){
              pixInd[0]=size[0]-1;
              pixInd[1]=0;
              L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
            }else{
              if(pixelIndex[1]>(size[1]-1)){
                pixInd[0]=size[0]-1;
                pixInd[1]=size[1]-1;
                L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
              }else{//pixelIndex[1] in range
                pixInd[0]=size[0]-1;
                pixInd[1]=pixelIndex[1];
                L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
              }
            }          
          }else{//pixelIndex[0] in range
            if(pixelIndex[1]<0){
              pixInd[0]=pixelIndex[0];
              pixInd[1]=0;
              L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
            }else{
              if(pixelIndex[1]>(size[1]-1)){
                pixInd[0]=pixelIndex[0];
                pixInd[1]=size[1]-1;
                L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
              }else{//pixelIndex[1] in range
                pixInd[0]=pixelIndex[0];
                pixInd[1]=pixelIndex[1];
                L[0] = ( (float)this->GetInput()->GetPixel(pixInd) );
              }
            }
          }
        }        
        
        //algorithm of Transform of Hermite order 3
        MRindex[0] = 0;
        L[1] = L[0] + MR->GetPixel(MRindex);
        L[2] = L[0] - MR->GetPixel(MRindex);
        MR->SetPixel(MRindex,L[0]);
          
        L[3] = L[1] + MC[0];
        L[4] = L[1] - MC[0]; 
        MC[0] = L[1];
        
        L[5] = L[2] + MC[1];
        MC[1] = L[2];

        MRindex[0] = 1;
        L[6] = L[3] + MR->GetPixel(MRindex);
        MR->SetPixel(MRindex,L[3]);

        MRindex[0] = 2;
        L[7] = (L[4] + MR->GetPixel(MRindex));
        MR->SetPixel(MRindex,L[4]);
        
        MRindex[0] = 3;
        L[8] = L[5] - MR->GetPixel(MRindex);
        MR->SetPixel(MRindex,L[5]);
                
        L[9] = (L[6] + MC[2]);
        MC[2] = L[6];
          
        L[10] = L[7] - MC[3];
        MC[3] = L[7];

        L[11] = (L[8] + MC[4]);
        MC[4] = L[8];
        
        MRindex[0] = 4;
        L[12] = (L[9] + MR->GetPixel(MRindex));
        L[13] = L[9] - MR->GetPixel(MRindex); 
        MR->SetPixel(MRindex,L[9]);
        
        MRindex[0] = 5;
        L[14] = (L[10] + MR->GetPixel(MRindex));
        L[15] = L[10] - MR->GetPixel(MRindex); 
        MR->SetPixel(MRindex,L[10]);
        
        MRindex[0] = 6;
        L[16] = (L[11] + MR->GetPixel(MRindex));
        L[17] = L[11] - MR->GetPixel(MRindex);
        MR->SetPixel(MRindex,L[11]);
                
        //normalized coefficients (with 1/(2^sums))  
        //    null order    
        pixC[2] =0;  
          coef->SetPixel( pixC,(L[12] + MC[5])/64 );  //00  
                  
        //    first order
        pixC[2] =1;  
          coef->SetPixel( pixC,(L[13] + MC[6])/32 );  //10    
        pixC[2] =2;  
          coef->SetPixel( pixC,(L[12] - MC[5])/32 );  //01  
                        
        //    second order
        pixC[2] =3;  
          coef->SetPixel( pixC,(L[16] + MC[9])/16 );  //20  
        pixC[2] =4;  
          coef->SetPixel( pixC,(L[13] - MC[6])/16 );  //11
        pixC[2] =5;  
          coef->SetPixel( pixC,(L[14] + MC[7])/16);  //02 
            
        //  third order
        pixC[2] =9;  
          coef->SetPixel( pixC,(L[14] - MC[7])/8 );  //03  
        pixC[2] =6;  
          coef->SetPixel( pixC,(L[17] + MC[10])/8 ); //30
        pixC[2] =8;  
          coef->SetPixel( pixC,(L[15] + MC[8])/8 );  //12  
        pixC[2] =7;  
          coef->SetPixel( pixC,(L[16] - MC[9])/8 );  //21    
        
        MC[5] = L[12];
        MC[6] = L[13];
        MC[7] = L[14];
        MC[8] = L[15];
        MC[9] = L[16];
        MC[10] = L[17];

      }
    }    
      
    //Average and rotated coefficients
    for(int i=0;i<10;i++){
      L[i] = 0.0;
      MC[i] = 0.0;
    }
    
    region2Size[0] = 10; 
    region2Size[1] = size[1]+5; 
    region2.SetSize( region2Size );
    MR->SetRegions(region2);
    MR->Allocate();
    MR->FillBuffer(0.0);
    
    for(int i=0;i<size[0]+5;i++){
      for(int j=0;j<size[1]+5;j++){
        //average
        pixC[0] =i; pixC[1] =j;
        MRindex[1] =j;          
        for(int k=0;k<10;k++){
          pixC[2] =k; 
          float pix = coef->GetPixel(pixC);
                  
          MRindex[0] = k;
          L[k] = pix + MR->GetPixel(MRindex);
          MR->SetPixel(MRindex,pix);
          
          pixelIndex[0] =i-4; 
          pixelIndex[1] =j-4;                   
          if((pixelIndex[0]>=0)&(pixelIndex[1]>=0)&
             (pixelIndex[0]<size[0])&(pixelIndex[1]<size[1])){  
             
            this->GetNthOutput(k)->SetPixel( pixelIndex,(L[k] + MC[k])/4 ); 
          }          
          MC[k]=L[k];
        }
        
        //rotation  
        pixelIndex[0] =i-4; 
        pixelIndex[1] =j-4;
        if((pixelIndex[0]>=0)&(pixelIndex[1]>=0)&
           (pixelIndex[0]<size[0])&(pixelIndex[1]<size[1])){  
                  
          //angle in radians
          float th = atan2( (float)this->GetNthOutput(2)->GetPixel
            (pixelIndex),(float)this->GetNthOutput(1)->GetPixel(pixelIndex) );      
          this->GetNthOutput(10)->SetPixel(pixelIndex,th);  
              
          float cth = cos(th);
          float sth = sin(th);
          
          //    null order = coefficien 00
        
          //    first order (of Coeffiecien 10)
          this->GetNthOutput(1+10)->SetPixel(pixelIndex, 
            (sth*this->GetNthOutput(2)->GetPixel(pixelIndex)) + 
            (cth*this->GetNthOutput(1)->GetPixel(pixelIndex)) ); //10      
          
          //    second order (of Coeffiecien 20)    
          this->GetNthOutput(2+10)->SetPixel(pixelIndex, 
            (sth*sth*this->GetNthOutput(5)->GetPixel(pixelIndex))+
            (1.4142*cth*sth*this->GetNthOutput(4)->GetPixel(pixelIndex))+
            (cth*cth*this->GetNthOutput(3)->GetPixel(pixelIndex)) );//20        
          
          //  third order  (of Coeffiecien 30)  
          this->GetNthOutput(3+10)->SetPixel(pixelIndex, 
            (sth*sth*sth*this->GetNthOutput(9)->GetPixel(pixelIndex))+
            (1.7321*cth*sth*sth*this->GetNthOutput(8)->GetPixel(pixelIndex))+
            (1.7321*cth*cth*sth*this->GetNthOutput(7)->GetPixel(pixelIndex))+
            (cth*cth*cth*this->GetNthOutput(6)->GetPixel(pixelIndex))); //30  
        }
      }
    }
    
    /*  
    //Normalize output values in [0-255], only for visualization  
    typedef itk::RescaleIntensityImageFilter< OutputImageType, OutputImageType > RescaleFilterType;
    RescaleFilterType::Pointer rescaleFilter = RescaleFilterType::New();
    rescaleFilter->SetOutputMinimum(0);
    rescaleFilter->SetOutputMaximum(255);

    for(int i=0;i<21;i++){
      rescaleFilter->SetInput(this->GetNthOutput(i));
      rescaleFilter->Update();  
      //this->GetNthOutput(i) = rescaleFilter->GetOutput();
      for(int k=0;k<size[0];k++){
        pixelIndex[0] =k;
        for(int j=0;j<size[1];j++){
          pixelIndex[1] =j;        
          this->GetNthOutput(i)->SetPixel
            ( pixelIndex, rescaleFilter->GetOutput()->GetPixel(pixelIndex) );
        }
      }
    }    
    */
  }
   
  template< class TInputImage, class TOutputImage>
  DataObject::Pointer HermiteImageFilter<TInputImage, TOutputImage>::MakeOutput()
  {
    OutputImageType::Pointer output;
    output = ( OutputImageType::New() ).GetPointer();

    return output.GetPointer();
  }

  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteImageFilter<TInputImage, TOutputImage>::
    GetNthOutput(int n)
  {
    return dynamic_cast< OutputImageType * >
      ( this->ProcessObject::GetOutput(n) );
  }
    
  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteImageFilter<TInputImage, TOutputImage>::
    GetCoef(int n)
  {
    return GetNthOutput(n);
  }

  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteImageFilter<TInputImage, TOutputImage>::
    GetRotCoef(int n)
  {
    return GetNthOutput(n+10);
  }
  
  template< class TInputImage, class TOutputImage>
  TOutputImage* HermiteImageFilter<TInputImage, TOutputImage>::
    GetAngle()
  {
    return GetNthOutput(10);
  }
 
}// end namespace
 
#endif