#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "opticalflowwidget.h"
#include "ui_opticalflowwidget.h"

#include "mmodewidget.h"
#include "ui_mmodewidget.h"

#include "Open4D.h"

#include "OpenVol.h"

#include "itkHermiteOpticalFlow.h"

#include <QVBoxLayout>

#include <vtkMetaImageReader.h>
#include <vtkVolumeRayCastMapper.h>
#include <vtkVolumeRayCastCompositeFunction.h>
#include <vtkColorTransferFunction.h>
#include <vtkCamera.h>
#include <vtkProperty.h>
#include <vtkImageFlip.h>
#include <vtkTransformPolyDataFilter.h>
#include <vtkPolyDataToImageStencil.h>
#include <vtkImageStencilData.h>
#include <vtkImageStencilToImage.h>
#include <vtkImageSobel2D.h>
#include <vtkMetaImageWriter.h>
#include <vtkBMPWriter.h>
#include <vtkParametricSpline.h>
#include <vtkParametricFunctionSource.h>

#include <itkVTKImageToImageFilter.h>
#include <itkImageToVTKImageFilter.h>
#include <itkBinaryContourImageFilter.h>
#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>
#include <itkImageFileReader.h>
#include <itkExtractImageFilter.h>

#include <vnl/vnl_matrix.h>
#include <vnl/vnl_vector.h>

const double MainWindow::axialElements[16] = {    
1, 0, 0, 0,
0, 1, 0, 0,
0, 0, 1, 0,
0, 0, 0, 1       
};

const double MainWindow::sagitalElements[16] = {  
0, 0, 1, 0,
1, 0, 0, 0,
0, 1, 0, 0,
0, 0, 0, 1               
};

const double MainWindow::coronalElements[16] = { 
1, 0, 0, 0,
0, 0, 1, 0,
0, 1, 0, 0,
0, 0, 0, 1        
};

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

  //initializing volume display widget
  volWidget = new QVTKWidget;
  QVBoxLayout *volLayout = new QVBoxLayout;
  volLayout->setContentsMargins(0, 0, 0, 0);
  volLayout->setSpacing(0);
  volLayout->QLayout::addWidget(volWidget);
  ui->volView->setLayout(volLayout);

  //initializing sagital display widget
  sagitalWidget = new QVTKWidget;
  QVBoxLayout *sagitalLayout = new QVBoxLayout;
  sagitalLayout->setContentsMargins(0, 0, 0, 0);
  sagitalLayout->setSpacing(0);
  sagitalLayout->QLayout::addWidget(sagitalWidget);
  ui->sagitalView->setLayout(sagitalLayout);

  //initializing axial display widget
  axialWidget = new QVTKWidget;
  QVBoxLayout *axialLayout = new QVBoxLayout;
  axialLayout->setContentsMargins(0, 0, 0, 0);
  axialLayout->setSpacing(0);
  axialLayout->QLayout::addWidget(axialWidget);
  ui->axialView->setLayout(axialLayout);

  //initializing coronal display widget
  coronalWidget = new QVTKWidget;
  QVBoxLayout *coronalLayout = new QVBoxLayout;
  coronalLayout->setContentsMargins(0, 0, 0, 0);
  coronalLayout->setSpacing(0);
  coronalLayout->QLayout::addWidget(coronalWidget);
  ui->coronalView->setLayout(coronalLayout);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::setNewClickedPosition(double x,double y){

  if(x==0&&y==0){

    //click outside the image
    std::cout<<"Please select a point inside the image"<<std::endl;
    return;

  }
  
  this->clickX = x;
  this->clickY = y;

  if(ui->rotCenterBtn->isChecked()){
    setNewRotationCenter();
  }

}

void MainWindow::setNewRotationCenter()
{

  if(ui->sagitalViewBtn->isChecked()){

    //change rotation center in selected view
    rotCenterSagital[0] = clickX;
    rotCenterSagital[1] = clickY;

    //change rotation center reference in selected view
    sagitalCenterRef->SetCenter(rotCenterSagital[0],rotCenterSagital[1],0);
    sagitalCenterRef->Update();
    sagitalCenterRefMapper->Update();

    std::cout<<"New rotation center: "<<rotCenterSagital[0]<<","<<rotCenterSagital[1]<<std::endl;

    //displaying new rotation center reference in selected view
    displaySagital();

  }else if(ui->axialViewBtn->isChecked()){

    //change rotation center in selected view
    rotCenterAxial[0] = clickX;
    rotCenterAxial[1] = clickY;

    //change rotation center reference in selected view
    axialCenterRef->SetCenter(rotCenterAxial[0],rotCenterAxial[1],0);
    axialCenterRef->Update();
    axialCenterRefMapper->Update();

    std::cout<<"New rotation center: "<<rotCenterAxial[0]<<","<<rotCenterAxial[1]<<std::endl;

    //displaying new rotation center reference in selected view
    displayAxial();

  }else if(ui->coronalViewBtn->isChecked()){

    //change rotation center in selected view
    rotCenterCoronal[0] = clickX;
    rotCenterCoronal[1] = clickY;

    //change rotation center reference in selected view
    coronalCenterRef->SetCenter(rotCenterCoronal[0],rotCenterCoronal[1],0);
    coronalCenterRef->Update();
    coronalCenterRefMapper->Update();

    std::cout<<"New rotation center: "<<rotCenterCoronal[0]<<","<<rotCenterCoronal[1]<<std::endl;

    //displaying new rotation center reference in selected view
    displayCoronal();

  }

}

void MainWindow::setSegmentedPath(vtkSmartPointer<vtkPolyData> anotation)
{
  if(ui->segmentBtn->isChecked()){
    
    if(!segmented){
      segmented = true;
      ui->saveSegBtn->setEnabled(true);

      //initializing segmented image
      segmentedImage = vtkSmartPointer<vtkImageData>::New();
      segmentedImage->DeepCopy(displayVolume);
      }

    vtkSmartPointer<vtkImageData> segmentedSlice;
    vtkSmartPointer<vtkTransform> transform;
    vtkSmartPointer<vtkPolyDataToImageStencil> dataToStencil = 
    vtkSmartPointer<vtkPolyDataToImageStencil>::New();
    vtkSmartPointer<vtkImageStencilToImage> stencilToImageFilter = 
    vtkSmartPointer<vtkImageStencilToImage>::New();

    if(ui->sagitalViewBtn->isChecked()){  
      segmentedSlice = reslicerSagital->GetOutput();
      transform = transformSagital;
    }else if(ui->axialViewBtn->isChecked()){
      segmentedSlice = reslicerAxial->GetOutput();
      transform = transformAxial;
    }else if(ui->coronalViewBtn->isChecked()){
      segmentedSlice = reslicerCoronal->GetOutput();
      transform = transformCoronal;
    }

    dataToStencil->SetInput(anotation);
    dataToStencil->SetOutputSpacing(segmentedSlice->GetSpacing());
    dataToStencil->SetOutputOrigin(displayVolume->GetOrigin());
    dataToStencil->SetOutputWholeExtent(segmentedSlice->GetWholeExtent());
    dataToStencil->Update();

    stencilToImageFilter->SetInput(dataToStencil->GetOutput());
    stencilToImageFilter->SetOutsideValue(0);
    stencilToImageFilter->SetInsideValue(255);
    stencilToImageFilter->SetOutputScalarTypeToUnsignedChar();
    stencilToImageFilter->Update();

    typedef itk::Image<unsigned char,2> ImageType;
    typedef itk::VTKImageToImageFilter<ImageType> ITKConverterType;
    ITKConverterType::Pointer itkConverter = ITKConverterType::New();

    itkConverter->SetInput(stencilToImageFilter->GetOutput());
    itkConverter->Update();

    typedef itk::BinaryContourImageFilter<ImageType,ImageType> ContourFilterType;
    ContourFilterType::Pointer contourFilter = ContourFilterType::New();

    contourFilter->SetInput(itkConverter->GetOutput());
    contourFilter->Update();

    typedef itk::ImageToVTKImageFilter<ImageType> VTKConverterType;
    VTKConverterType::Pointer vtkConverter = VTKConverterType::New();
    vtkConverter->SetInput(contourFilter->GetOutput());
    vtkConverter->Update();

    vtkSmartPointer<vtkImageData> edgeImage = vtkConverter->GetOutput();

    int *imageSize = edgeImage->GetDimensions();
    double *spacingImage = edgeImage->GetSpacing();

    for(int x = 0; x<imageSize[0]; x++){
        for(int y = 0; y<imageSize[1]; y++){

          unsigned char * imagePixel = static_cast<unsigned char *> 
        (edgeImage->GetScalarPointer(x,y,0));

          if(imagePixel[0]==255){
            
            float tempPoint[3];

            if(ui->sagitalViewBtn->isChecked()){  
              tempPoint[0] = 0;
              tempPoint[1] = spacingImage[1]*x;
              tempPoint[2] = spacingImage[0]*y; 
            }else if(ui->axialViewBtn->isChecked()){
              tempPoint[0] = spacingImage[0]*x;
              tempPoint[1] = spacingImage[1]*y;
              tempPoint[2] = 0;
            }else if(ui->coronalViewBtn->isChecked()){
              tempPoint[0] = spacingImage[0]*x;
              tempPoint[1] = 0;
              tempPoint[2] = spacingImage[1]*y;
            }

            const float point[3] = {tempPoint[0],
                            tempPoint[1],
                  tempPoint[2]};

            float transformedPoint[3];
            transform->TransformPoint(point,transformedPoint);
            
            int voxel[3];
            voxel[0] = vtkMath::Floor((transformedPoint[0]/spacing[0]) + 0.5);
            voxel[1] = vtkMath::Floor((transformedPoint[1]/spacing[1]) + 0.5);
            voxel[2] = vtkMath::Floor((transformedPoint[2]/spacing[2]) + 0.5);

            if( (-1<voxel[0]&&voxel[0]<dimensions[0])&&
        (-1<voxel[1]&&voxel[1]<dimensions[1])&&
        (-1<voxel[2]&&voxel[2]<dimensions[2]) ){

              unsigned char * volumeVoxel = static_cast<unsigned char *> 
          (segmentedImage->GetScalarPointer(voxel[0],
                                            voxel[1],
                          voxel[2]));
              volumeVoxel[0] = 255;
            }
          }
        }
    }

    vtkSmartPointer<vtkMetaImageWriter> writer = 
    vtkSmartPointer<vtkMetaImageWriter>::New();
    writer->SetFileName("C:/Users/Fabian/Desktop/slice.mhd");
    writer->SetRAWFileName("C:/Users/Fabian/Desktop/slice.raw");
    writer->SetInput(segmentedImage);
    writer->Write();
  }
  
  if(ui->mModeBtn->isChecked()){
    if(anotation->GetNumberOfPoints() == 2){
      std::cout<<std::endl;
      std::cout<<"M-Mode "<<std::endl;
  
      //max. number of points in spline = sizeX + sizeY
      int nPnt = ( imgSecVTK[0]->GetDimensions()[0] ) + 
             ( imgSecVTK[0]->GetDimensions()[1] );

      double *spacing =imgSecVTK[0]->GetSpacing();
      double *origin =imgSecVTK[0]->GetOrigin();
      
      mMode = vtkSmartPointer<vtkImageData>::New();
      mMode->SetOrigin(origin[0],origin[1],origin[2]);
      mMode->SetSpacing(spacing[0],spacing[1],0);
      mMode->SetScalarTypeToUnsignedChar();
      mMode->SetNumberOfScalarComponents(1);

      //spline 
      vtkSmartPointer<vtkParametricSpline> spline = 
      vtkSmartPointer<vtkParametricSpline>::New();
      spline->ClosedOff();
      spline->SetPoints(anotation->GetPoints());
      vtkSmartPointer<vtkParametricFunctionSource> functionSource = 
      vtkSmartPointer<vtkParametricFunctionSource>::New();
      functionSource->SetParametricFunction(spline);
      functionSource->SetUResolution(nPnt);
      functionSource->SetVResolution(nPnt);
      functionSource->Update();

      //X,Y,Z positions of points
      vtkSmartPointer<vtkPoints> pts = 
      vtkSmartPointer<vtkPoints>::New();

      //new number of points
      vtkIdType n = 0;

      //previous point
      int point2[2];

      //delete repeated points and transform coordinates to integer 
      for(vtkIdType i=0;i<functionSource->GetOutput()->GetNumberOfPoints();i++)
      {
        //actual point
        int point[2];
        point[0] = vtkMath::Floor((functionSource->GetOutput()->
          GetPoints()->GetPoint(i)[0]/spacing[0]) + 0.5);
        point[1] = vtkMath::Floor((functionSource->GetOutput()->
          GetPoints()->GetPoint(i)[1]/spacing[1]) + 0.5);
        
        if( (-1<point[0]&&point[0]<dimensions[0]) && 
            (-1<point[1]&&point[1]<dimensions[1]) ){
          //first point OR new point is not first point and not equal of previous point
          if(  (i==0)||((i!=0)&&
                 ( (point[0]!=point2[0])||(point[1]!=point2[1]) )) ){
            point2[0] = point[0];
            point2[1] = point[1];
            pts->SetDataTypeToInt();
            pts->InsertPoint( n, point2[0], point2[1], 0 );
            n++;
          }          
        }
      }
      
      //dimensions of image M-Mode
      mMode->SetDimensions(imgSecVTK.size(),n,1);
      mMode->Update();

      //set values to image of M-Mode
      for(int x = 0; x < imgSecVTK.size(); x++)
      {
        for(int y = 0; y < int(n); y++)
        {
          unsigned char * valueMmode = static_cast<unsigned char *> 
        (mMode->GetScalarPointer(x,y,0));
          valueMmode[0] = static_cast<unsigned char *> 
        (imgSecVTK[x]->GetScalarPointer(int(pts->GetPoint(vtkIdType(n-y-1))[0]),
                                        int(pts->GetPoint(vtkIdType(n-y-1))[1]),
                        0))[0];
        }
      }
      mMode->Update();
      mModeFlag = true;
      
      //removing m-mode interactor style from selected view 
      mModeStyle->clearTracer();
      viewerAxial->GetRenderer()->AddActor(axialCenterRefActor);
      viewerAxial->GetRenderWindow()->GetInteractor()->
      SetInteractorStyle(imageStyleAxial);
      
      //no checked boton M-Mode
      ui->mModeBtn->setChecked(false);

      //initialize values      
      periodMmode[0] = 0;
      periodMmode[1] = 0;
      ciclesMmode = 1;
      
      //Window M-Mode
      MmodeWidget *widget = new MmodeWidget;
      widget->setWindowTitle("M-Mode");
      widget->setImageData(mMode);
      widget->setMainWindow(this);
      widget->show();  
    }else{
      std::cout<<"M-Mode need one rect line, please pick only two poins"<<std::endl;
      
      //removing m-mode interactor style from selected view 
      mModeStyle->clearTracer();
      viewerAxial->GetRenderer()->AddActor(axialCenterRefActor);
      viewerAxial->GetRenderWindow()->GetInteractor()->
      SetInteractorStyle(imageStyleAxial);
      
      //no checked boton M-Mode
      ui->mModeBtn->setChecked(false);
    }
  }

}

void MainWindow::setPeriod(int p0,int p1,int cicles){
  //save selected period and cicles
  periodMmode[0]=p0;
  periodMmode[1]=p1;
  ciclesMmode=cicles;
}

void MainWindow::displayVol()
{
  volumeProperty = vtkSmartPointer<vtkVolumeProperty>::New();

  //setting volume rendering methods and properties
    vtkSmartPointer<vtkVolumeRayCastCompositeFunction> rayCastFunction = vtkSmartPointer<vtkVolumeRayCastCompositeFunction>::New();

    volumeScalarOpacity = vtkSmartPointer<vtkPiecewiseFunction>::New();
    volumeScalarOpacity->AddPoint(0,1.00);
    volumeScalarOpacity->AddPoint(255,1.00);
    volumeScalarOpacity->Update();
    volumeProperty->SetScalarOpacity(volumeScalarOpacity);

    vtkSmartPointer<vtkColorTransferFunction> volumeColor = vtkSmartPointer<vtkColorTransferFunction>::New();
    volumeColor->AddRGBPoint(0,0.0,0.0,0.0);
    volumeColor->AddRGBPoint(64,0.25,0.25,0.25);
    volumeColor->AddRGBPoint(128,0.5,0.5,0.5);
    volumeColor->AddRGBPoint(192,0.75,0.75,0.75);
    volumeColor->AddRGBPoint(255,1.0,1.0,1.0);
    volumeProperty->SetColor(volumeColor);

    vtkSmartPointer<vtkVolumeRayCastCompositeFunction> compositeFunction = vtkSmartPointer<vtkVolumeRayCastCompositeFunction>::New();

    vtkSmartPointer<vtkVolumeRayCastMapper> volumeMapper = vtkSmartPointer<vtkVolumeRayCastMapper>::New();
    volumeMapper->SetVolumeRayCastFunction(compositeFunction);
    volumeMapper->CroppingOff();
    volumeMapper->SetInput(displayVolume);

  //setting volume rendering data
    volume = vtkSmartPointer<vtkVolume>::New();
    volume->SetMapper(volumeMapper);
    volume->SetOrigin(0,0,0);
    volume->SetProperty(volumeProperty);
    volume->Update();

  //setting 3D scene properties
  renderer = vtkSmartPointer<vtkRenderer>::New();
    renderer->SetBackground(183.0/255.0,197.0/255.0,253.0/255.0);
    renderer->AddVolume(volume);
    renderer->GetActiveCamera()->SetFocalPoint(volume->GetCenter());
    renderer->GetActiveCamera()->Roll(90);
    
    double cameraPos[3];
    cameraPos[0] = volume->GetCenter()[0] + volume->GetMaxXBound() + 30;
    cameraPos[1] = volume->GetCenter()[1];
    cameraPos[2] = volume->GetCenter()[2];
    
    renderer->GetActiveCamera()->SetPosition(cameraPos);

  //displaying volume
    renwin = vtkSmartPointer<vtkRenderWindow>::New();
    renwin->AddRenderer(renderer);
    
    volWidget->SetRenderWindow(renwin);
    std::cout<<std::endl;
    std::cout<<"Displaying volume"<<std::endl<<std::endl;
    renwin->Render();
}

void MainWindow::setRenderingData()
{

  //Displayed volume
  displayVolume = vtkSmartPointer<vtkImageData>::New();
  displayVolume->DeepCopy(volumeData);

  //initializing sagital 2D view
  imageStyleSagital = vtkSmartPointer<vtkInteractorStyleImage>::New();
  viewerSagital = vtkSmartPointer<vtkImageViewer2>::New();
  imageActorSagital = vtkSmartPointer<vtkImageActor>::New();
    sagitalWidget->SetRenderWindow(viewerSagital->GetRenderWindow());
  imageActorSagital = viewerSagital->GetImageActor();
  imageActorSagital->InterpolateOff();
  viewerSagital->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleSagital);

  //initializing axial 2D view
  imageStyleAxial = vtkSmartPointer<vtkInteractorStyleImage>::New();
  viewerAxial = vtkSmartPointer<vtkImageViewer2>::New();
  imageActorAxial = vtkSmartPointer<vtkImageActor>::New();
    axialWidget->SetRenderWindow(viewerAxial->GetRenderWindow());
  imageActorAxial = viewerAxial->GetImageActor();
  imageActorAxial->InterpolateOff();
  viewerAxial->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleAxial);

  //initializing coronal 2D view
  imageStyleCoronal = vtkSmartPointer<vtkInteractorStyleImage>::New();
  viewerCoronal = vtkSmartPointer<vtkImageViewer2>::New();
  imageActorCoronal = vtkSmartPointer<vtkImageActor>::New();
    coronalWidget->SetRenderWindow(viewerCoronal->GetRenderWindow());
  imageActorCoronal = viewerCoronal->GetImageActor();
  imageActorCoronal->InterpolateOff();
  viewerCoronal->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleCoronal);

  segmented = false;
}

void MainWindow::setSlicesData()
{
  std::cout<<"Displaying Slices"<<std::endl;

  //getting volume properties
  displayVolume->GetSpacing(spacing);
  displayVolume->GetOrigin(origin);
    displayVolume->GetDimensions(dimensions);

  //setting center of volume 
  centerSlice[0] = floor(dimensions[0]*0.5)-1;
  centerSlice[1] = floor(dimensions[1]*0.5)-1;
  centerSlice[2] = floor(dimensions[2]*0.5)-1;

  positionCenter[0] = origin[0] + spacing[0] * centerSlice[0];
  positionCenter[1] = origin[1] + spacing[1] * centerSlice[1]; 
    positionCenter[2] = origin[2] + spacing[2] * centerSlice[2];

  //configureation of 2D views
  configSagitalView();
  configAxialView();
  configCoronalView();

  //setting initial values for sliders
  ui->rotXSld->setRange(-90,90);
  ui->rotXSld->setTickInterval(1);
  ui->rotXSld->setValue(0);

  ui->rotYSld->setRange(-90,90);
  ui->rotYSld->setTickInterval(1);
  ui->rotYSld->setValue(0);

  ui->rotZSld->setRange(-90,90);
  ui->rotZSld->setTickInterval(1);
  ui->rotZSld->setValue(0);

  ui->sliceSld->setRange(0,dimensions[2]-1);
  ui->sliceSld->setTickInterval(1);
  ui->sliceSld->setValue(centerSlice[2]);

  addActorsAxialView();

  //displaying 2D views
  displaySagital();
  displayCoronal();
}

void MainWindow::configSagitalView()
{
  //setting initial variable values
  sliceSagital = centerSlice[0];
  angleXSagital = 0;
  angleYSagital = 0;
  angleZSagital = 0; 
  
  //initializing rotation center
  rotCenterSagital[0] = positionCenter[1]-origin[1];
  rotCenterSagital[1] = positionCenter[2]-origin[2];

  //initializing slicer data
  resliceAxesSagital = vtkSmartPointer<vtkMatrix4x4>::New();
  reslicerSagital = vtkSmartPointer<vtkImageReslice>::New();
  transformSagital = vtkSmartPointer<vtkTransform>::New();

  transformSagital->PostMultiply();
  transformSagital->Translate(positionCenter[0],origin[1],origin[2]);

  resliceAxesSagital->DeepCopy(sagitalElements);

  reslicerSagital->SetInput(displayVolume);
  reslicerSagital->SetOutputDimensionality(2);
  reslicerSagital->SetResliceAxes(resliceAxesSagital);
  reslicerSagital->SetResliceTransform(transformSagital);
  reslicerSagital->SetInterpolationModeToLinear();
  
  //setting visual references for axial and coronal slices
  sagitalRefInAxialView = vtkSmartPointer<vtkLineSource>::New();
  sagitalRefInAxialViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  sagitalRefInAxialViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Axial[3] = {sliceSagital*spacing[0],dimensions[1]*spacing[1],0.0};
  double p1Axial[3] = {sliceSagital*spacing[0],0.0,0.0};
  
  sagitalRefInAxialView->SetPoint1(p0Axial);
  sagitalRefInAxialView->SetPoint2(p1Axial);
  sagitalRefInAxialView->Update();
  sagitalRefInAxialViewMapper->SetInputConnection(sagitalRefInAxialView->GetOutputPort());
  sagitalRefInAxialViewActor->SetMapper(sagitalRefInAxialViewMapper);
  sagitalRefInAxialViewActor->GetProperty()->SetLineWidth(1);
  sagitalRefInAxialViewActor->GetProperty()->SetColor(0,162,232);

  sagitalRefInCoronalView = vtkSmartPointer<vtkLineSource>::New();
  sagitalRefInCoronalViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  sagitalRefInCoronalViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Coronal[3] = {sliceSagital*spacing[0],dimensions[2]*spacing[2],0.0};
  double p1Coronal[3] = {sliceSagital*spacing[0],0.0,0.0};
  
  sagitalRefInCoronalView->SetPoint1(p0Coronal);
  sagitalRefInCoronalView->SetPoint2(p1Coronal);
  sagitalRefInCoronalView->Update();
  sagitalRefInCoronalViewMapper->SetInputConnection(sagitalRefInCoronalView->GetOutputPort());
  sagitalRefInCoronalViewActor->SetMapper(sagitalRefInCoronalViewMapper);
  sagitalRefInCoronalViewActor->GetProperty()->SetLineWidth(1);
  sagitalRefInCoronalViewActor->GetProperty()->SetColor(0,162,232);

  //seting visual reference for rotation center
  sagitalCenterRef = vtkSmartPointer<vtkRegularPolygonSource>::New();
  sagitalCenterRefMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  sagitalCenterRefActor = vtkSmartPointer<vtkActor>::New();
 
  sagitalCenterRef->SetNumberOfSides(50);
  sagitalCenterRef->SetRadius(0.5);
  sagitalCenterRef->SetCenter(rotCenterSagital[0],rotCenterSagital[1],0);

  sagitalCenterRefMapper->SetInputConnection(sagitalCenterRef->GetOutputPort());;
  sagitalCenterRefActor->SetMapper(sagitalCenterRefMapper);
  sagitalCenterRefActor->GetProperty()->SetColor(1.0,0.59,0.08);
}

void MainWindow::configAxialView()
{
  //setting initial variable values
  sliceAxial = centerSlice[2];
  angleXAxial = 0;
  angleYAxial = 0;
  angleZAxial = 0; 

  //initializing rotation center
  rotCenterAxial[0] = positionCenter[0]-origin[0];
  rotCenterAxial[1] = positionCenter[1]-origin[1];

  //initializing slicer data
  resliceAxesAxial = vtkSmartPointer<vtkMatrix4x4>::New();
  reslicerAxial = vtkSmartPointer<vtkImageReslice>::New();
  transformAxial = vtkSmartPointer<vtkTransform>::New();

  transformAxial->PostMultiply();
  transformAxial->Translate(origin[0],origin[1],positionCenter[2]);

  //transformAxial->Translate(-(rotCenterAxial[0]),-(rotCenterAxial[1]),-(sliceAxial*spacing[2]));
  //transformAxial->RotateZ(180);
  //transformAxial->Translate((rotCenterAxial[0]),(rotCenterAxial[1]),(sliceAxial*spacing[2]));

  resliceAxesAxial->DeepCopy(axialElements);

  reslicerAxial->SetInput(displayVolume);
  reslicerAxial->SetOutputDimensionality(2);
  reslicerAxial->SetResliceAxes(resliceAxesAxial);
  reslicerAxial->SetResliceTransform(transformAxial);
  reslicerAxial->SetInterpolationModeToLinear();

  //setting visual references for sagital and coronal slices
  axialRefInSagitalView = vtkSmartPointer<vtkLineSource>::New();
  axialRefInSagitalViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  axialRefInSagitalViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Sagital[3] = {dimensions[1]*spacing[1],sliceAxial*spacing[2],0.0};
  double p1Sagital[3] = {0.0,sliceAxial*spacing[2],0.0};
  
  axialRefInSagitalView->SetPoint1(p0Sagital);
  axialRefInSagitalView->SetPoint2(p1Sagital);
  axialRefInSagitalView->Update();
  axialRefInSagitalViewMapper->SetInputConnection(axialRefInSagitalView->GetOutputPort());
  axialRefInSagitalViewActor->SetMapper(axialRefInSagitalViewMapper);
  axialRefInSagitalViewActor->GetProperty()->SetLineWidth(1);
  axialRefInSagitalViewActor->GetProperty()->SetColor(237,0,0);

  axialRefInCoronalView = vtkSmartPointer<vtkLineSource>::New();
  axialRefInCoronalViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  axialRefInCoronalViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Coronal[3] = {dimensions[0]*spacing[0],sliceAxial*spacing[2],0.0};
  double p1Coronal[3] = {0.0,sliceAxial*spacing[2],0.0};
  
  axialRefInCoronalView->SetPoint1(p0Coronal);
  axialRefInCoronalView->SetPoint2(p1Coronal);
  axialRefInCoronalView->Update();
  axialRefInCoronalViewMapper->SetInputConnection(axialRefInCoronalView->GetOutputPort());
  axialRefInCoronalViewActor->SetMapper(axialRefInCoronalViewMapper);
  axialRefInCoronalViewActor->GetProperty()->SetLineWidth(1);
  axialRefInCoronalViewActor->GetProperty()->SetColor(237,0,0);

  //seting visual reference for rotation center
  axialCenterRef = vtkSmartPointer<vtkRegularPolygonSource>::New();
  axialCenterRefMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  axialCenterRefActor = vtkSmartPointer<vtkActor>::New();
 
  axialCenterRef->SetNumberOfSides(50);
  axialCenterRef->SetRadius(0.5);
  axialCenterRef->SetCenter(rotCenterAxial[0],rotCenterAxial[1],0);

  axialCenterRefMapper->SetInputConnection(axialCenterRef->GetOutputPort());;
  axialCenterRefActor->SetMapper(axialCenterRefMapper);
  axialCenterRefActor->GetProperty()->SetColor(1.0,0.59,0.08);
}

void MainWindow::configCoronalView()
{
  //setting initial variable values
  sliceCoronal = centerSlice[1];
  angleXCoronal = 0;
  angleYCoronal = 0;
  angleZCoronal = 0; 

  //initializing rotation center
  rotCenterCoronal[0] = positionCenter[0]-origin[0];
  rotCenterCoronal[1] = positionCenter[2]-origin[2];

  //initializing slicer data
  resliceAxesCoronal = vtkSmartPointer<vtkMatrix4x4>::New();
  reslicerCoronal = vtkSmartPointer<vtkImageReslice>::New();
  transformCoronal = vtkSmartPointer<vtkTransform>::New();

  transformCoronal->PostMultiply();
  transformCoronal->Translate(origin[0],positionCenter[1],origin[2]);

  resliceAxesCoronal->DeepCopy(coronalElements);

  reslicerCoronal->SetInput(displayVolume);
  reslicerCoronal->SetOutputDimensionality(2);
  reslicerCoronal->SetResliceAxes(resliceAxesCoronal);
  reslicerCoronal->SetResliceTransform(transformCoronal);
  reslicerCoronal->SetInterpolationModeToLinear();

  //setting visual references for sagital and axial slices
  coronalRefInSagitalView = vtkSmartPointer<vtkLineSource>::New();
  coronalRefInSagitalViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  coronalRefInSagitalViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Sagital[3] = {sliceCoronal*spacing[1],dimensions[2]*spacing[2],0.0};
  double p1Sagital[3] = {sliceCoronal*spacing[1],0.0,0.0};
  
  coronalRefInSagitalView->SetPoint1(p0Sagital);
  coronalRefInSagitalView->SetPoint2(p1Sagital);
  coronalRefInSagitalView->Update();
  coronalRefInSagitalViewMapper->SetInputConnection(coronalRefInSagitalView->GetOutputPort());
  coronalRefInSagitalViewActor->SetMapper(coronalRefInSagitalViewMapper);
  coronalRefInSagitalViewActor->GetProperty()->SetLineWidth(1);
  coronalRefInSagitalViewActor->GetProperty()->SetColor(237,128,0);

  coronalRefInAxialView = vtkSmartPointer<vtkLineSource>::New();
  coronalRefInAxialViewMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  coronalRefInAxialViewActor = vtkSmartPointer<vtkActor>::New();

  double p0Axial[3] = {dimensions[0]*spacing[0],sliceCoronal*spacing[1],0.0};
  double p1Axial[3] = {0.0,sliceCoronal*spacing[1],0.0};
  
  coronalRefInAxialView->SetPoint1(p0Axial);
  coronalRefInAxialView->SetPoint2(p1Axial);
  coronalRefInAxialView->Update();
  coronalRefInAxialViewMapper->SetInputConnection(coronalRefInAxialView->GetOutputPort());
  coronalRefInAxialViewActor->SetMapper(coronalRefInAxialViewMapper);
  coronalRefInAxialViewActor->GetProperty()->SetLineWidth(1);
  coronalRefInAxialViewActor->GetProperty()->SetColor(237,128,0);

  //seting visual reference for rotation center
  coronalCenterRef = vtkSmartPointer<vtkRegularPolygonSource>::New();
  coronalCenterRefMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
  coronalCenterRefActor = vtkSmartPointer<vtkActor>::New();
 
  coronalCenterRef->SetNumberOfSides(50);
  coronalCenterRef->SetRadius(0.5);
  coronalCenterRef->SetCenter(rotCenterCoronal[0],rotCenterCoronal[1],0);

  coronalCenterRefMapper->SetInputConnection(coronalCenterRef->GetOutputPort());;
  coronalCenterRefActor->SetMapper(coronalCenterRefMapper);
  coronalCenterRefActor->GetProperty()->SetColor(1.0,0.59,0.08);
}

void MainWindow::addActorsSagitalView()
{
  //add center reference actor
  viewerSagital->GetRenderer()->AddActor(sagitalCenterRefActor);

  //remove reference actors from other slices
  viewerSagital->GetRenderer()->RemoveActor(axialRefInSagitalViewActor);
  viewerSagital->GetRenderer()->RemoveActor(coronalRefInSagitalViewActor);

  //add reference to other slices
  if(ui->axialViewBtn->isChecked()){
    viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
    viewerAxial->GetRenderer()->AddActor(sagitalRefInAxialViewActor);
    viewerAxial->GetRenderer()->AddActor(coronalRefInAxialViewActor);
  }else if(ui->coronalViewBtn->isChecked()){
    viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);
    viewerCoronal->GetRenderer()->AddActor(sagitalRefInCoronalViewActor);
    viewerCoronal->GetRenderer()->AddActor(axialRefInCoronalViewActor);
  }else{
    viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
    viewerAxial->GetRenderer()->AddActor(sagitalRefInAxialViewActor);
    viewerAxial->GetRenderer()->AddActor(coronalRefInAxialViewActor);
    viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);
    viewerCoronal->GetRenderer()->AddActor(sagitalRefInCoronalViewActor);
    viewerCoronal->GetRenderer()->AddActor(axialRefInCoronalViewActor);
  }
}

void MainWindow::addActorsAxialView()
{
  //add center reference actor
  viewerAxial->GetRenderer()->AddActor(axialCenterRefActor);

  //remove reference actors from other slices
  viewerAxial->GetRenderer()->RemoveActor(sagitalRefInAxialViewActor);
  viewerAxial->GetRenderer()->RemoveActor(coronalRefInAxialViewActor);

  //add reference to other slices
  if(ui->sagitalViewBtn->isChecked()){
    viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);
    viewerSagital->GetRenderer()->AddActor(axialRefInSagitalViewActor);
    viewerSagital->GetRenderer()->AddActor(coronalRefInSagitalViewActor);
  }else if(ui->coronalViewBtn->isChecked()){
    viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);
    viewerCoronal->GetRenderer()->AddActor(sagitalRefInCoronalViewActor);
    viewerCoronal->GetRenderer()->AddActor(axialRefInCoronalViewActor);
  }else{
    viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);
    viewerSagital->GetRenderer()->AddActor(axialRefInSagitalViewActor);
    viewerSagital->GetRenderer()->AddActor(coronalRefInSagitalViewActor);
    viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);
    viewerCoronal->GetRenderer()->AddActor(axialRefInCoronalViewActor);
    viewerCoronal->GetRenderer()->AddActor(sagitalRefInCoronalViewActor);
  }
}

void MainWindow::addActorsCoronalView()
{
  //add center reference actor
  viewerCoronal->GetRenderer()->AddActor(coronalCenterRefActor);

  //remove reference actors from other slices
  viewerCoronal->GetRenderer()->RemoveActor(sagitalRefInCoronalViewActor);
  viewerCoronal->GetRenderer()->RemoveActor(axialRefInCoronalViewActor);

  //add reference to other slices
  if(ui->axialViewBtn->isChecked()){
    viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
    viewerAxial->GetRenderer()->AddActor(sagitalRefInAxialViewActor);
    viewerAxial->GetRenderer()->AddActor(coronalRefInAxialViewActor);
  }else if(ui->sagitalViewBtn->isChecked()){
    viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);
    viewerSagital->GetRenderer()->AddActor(axialRefInSagitalViewActor);
    viewerSagital->GetRenderer()->AddActor(coronalRefInSagitalViewActor);
  }else{
    viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
    viewerAxial->GetRenderer()->AddActor(sagitalRefInAxialViewActor);
    viewerAxial->GetRenderer()->AddActor(coronalRefInAxialViewActor);
    viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);
    viewerSagital->GetRenderer()->AddActor(axialRefInSagitalViewActor);
    viewerSagital->GetRenderer()->AddActor(coronalRefInSagitalViewActor);
  }
}

void MainWindow::removeActorsSagitalView()
{
  //remove actors from all views
  viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);
  viewerAxial->GetRenderer()->RemoveActor(sagitalRefInAxialViewActor);
  viewerCoronal->GetRenderer()->RemoveActor(sagitalRefInCoronalViewActor);
}

void MainWindow::removeActorsAxialView()
{
  //remove actors from all views
  viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
  viewerSagital->GetRenderer()->RemoveActor(axialRefInSagitalViewActor);
  viewerCoronal->GetRenderer()->RemoveActor(axialRefInCoronalViewActor);
}

void MainWindow::removeActorsCoronalView()
{
  //remove actors from all views
  viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);
  viewerSagital->GetRenderer()->RemoveActor(coronalRefInSagitalViewActor);
  viewerAxial->GetRenderer()->RemoveActor(coronalRefInAxialViewActor);
}

void MainWindow::displaySagital()
{
  //updating slice image
  reslicerSagital->Update();
  sliceImageSagital = reslicerSagital->GetOutput();
  sliceImageSagital->Update();
  viewerSagital->SetInput(sliceImageSagital);
    viewerSagital->Render();

  //updating visual references
  viewerAxial->Render();
  viewerCoronal->Render();
}

void MainWindow::displayAxial()
{
  //updating slice image
  reslicerAxial->Update();
  sliceImageAxial = reslicerAxial->GetOutput();
  sliceImageAxial->Update();
  viewerAxial->SetInput(sliceImageAxial);
    viewerAxial->Render();

  //updating visual references
  viewerSagital->Render();
  viewerCoronal->Render();
}

void MainWindow::displayCoronal()
{
  //updating slice image
  reslicerCoronal->Update();
  sliceImageCoronal = reslicerCoronal->GetOutput();
  sliceImageCoronal->Update();
  viewerCoronal->SetInput(sliceImageCoronal);
    viewerCoronal->Render();

  //updating visual references
  viewerSagital->Render();
  viewerAxial->Render();
}

void MainWindow::openMHD()
{
  std::cout<<"Loading Volume"<<std::endl;

  //initializing reader
    this->volumeFilename = QFileDialog::getOpenFileName(this, tr("Open Volume .mhd"),
        QDir::currentPath(), tr("Volume Files (*.mhd)"));
    
  vtkSmartPointer<vtkMetaImageReader> reader = vtkSmartPointer<vtkMetaImageReader>::New();
    reader->SetFileName(volumeFilename.toAscii().data());
    reader->Update();

    volumeData = reader->GetOutput();

  //displaying volume
  setRenderingData();
  displayVol();
  setSlicesData();
}

void MainWindow::openVol()
{
  std::cout<<"Loading Volume"<<std::endl;

  //initializing reader
    this->volumeFilename = QFileDialog::getOpenFileName(this, tr("Open Volume .vol"),
        QDir::currentPath(), tr("Volume Files (*.vol)"));
    
  OpenVol * reader = OpenVol::New();
    reader->SetFilename(volumeFilename);
  reader->Update();

  vtkSmartPointer<vtkImageFlip> volFlipY = vtkSmartPointer<vtkImageFlip>::New();
  volFlipY->SetInput(reader->GetOutput());
  volFlipY->SetFilteredAxis(1);
  volFlipY->Update();

  volumeData = volFlipY->GetOutput();

  //displaying volume
  setRenderingData();
  displayVol();
  setSlicesData();
}

void MainWindow::open4D()
{
  std::cout<<"Loading Volume 4D"<<std::endl;

  //initializing reader
    this->volumeFilename = QFileDialog::getOpenFileName(this, 
    tr("Open Volume 4D .vol"),QDir::currentPath(), 
    tr("Volume Files (*.vol)"));

  Open4D * reader = Open4D::New();
  reader->SetFilename(volumeFilename);
  reader->Update();
  
  //Return volume 4d  
  vol4 = Image4Type::New();
  vol4->Allocate();
  vol4 = (reader->GetOutput());

  Image4Type::RegionType inputRegion = 
    vol4.GetPointer()->GetLargestPossibleRegion();
  Image4Type::SizeType vol4size = inputRegion.GetSize();
  Image4Type::IndexType vol4start = inputRegion.GetIndex();

  int nOfSequences = vol4size[3]-1;

  ui->sequenceSld->setEnabled(true);
  ui->sequenceSld->setRange(0,nOfSequences);

  //enable box FixPlane
  ui->fixPlaneBox->setEnabled(true);
  //box FixPlane not checked
  ui->fixPlaneBox->setChecked(false);

  //disable boton M-Mode
  ui->mModeBtn->setDisabled(true);
  mModeFlag = false;

  //initialize period of M-Mode in 0
  periodMmode[0] = 0;
  periodMmode[1] = 0;

  //initialize times of repited M-Mode in 1
  ciclesMmode = 1;

  //disable boton moveLV (move in left ventricle)
  ui->opticalFlowBtn->setDisabled(true);

  //enable rotation
  ui->rotXSld->setEnabled(true);
  ui->rotYSld->setEnabled(true);
  ui->rotZSld->setEnabled(true);

  //enable slice
  ui->sliceSld->setEnabled(true);
  
  //collapse sequence, 4D->3D
  vol4size[3] = 0;
  
  //create itk 3d image type
  typedef itk::Image<unsigned char, 3> ImageType;
  
  volumesVTK = std::vector< vtkSmartPointer<vtkImageData> > 
    (nOfSequences+1);
  
  for (int s=0; s<=nOfSequences; s++) {
    //sequence number
    vol4start[3] = s;

    //desire Region
    Image4Type::RegionType desiredRegion;
    desiredRegion.SetSize(  vol4size  );
    desiredRegion.SetIndex( vol4start );

    //assign first volume of sequence in itk 
    typedef itk::ExtractImageFilter< Image4Type, ImageType  >  
    ExtractFilterType;
    ExtractFilterType::Pointer filter = ExtractFilterType::New();
    filter->InPlaceOn();
    filter->SetDirectionCollapseToSubmatrix();
    //Set desire region to filter
    filter->SetExtractionRegion( desiredRegion );
    filter->SetInput( vol4.GetPointer() );
    filter->Update();

    //transform volume itk to volume vtk
    typedef itk::ImageToVTKImageFilter<ImageType> itkVtkConverter;
    itkVtkConverter::Pointer converter = itkVtkConverter::New();
    converter->SetInput(filter->GetOutput());
    converter->Update();

    vtkSmartPointer<vtkImageFlip> volFlipY = 
    vtkSmartPointer<vtkImageFlip>::New();
    volFlipY->SetInput(converter->GetOutput());
    volFlipY->SetFilteredAxis(1);
    volFlipY->Update();

    volumesVTK[s] = vtkSmartPointer<vtkImageData>::New();
    volumesVTK[s] = volFlipY->GetOutput();
    volumesVTK[s]->Update();
  }

  volumeData = volumesVTK[0];

  //displaying volume
  setRenderingData();
  displayVol();
  setSlicesData();  
}

void MainWindow::fixPlaneBoxClicked(bool value){
  if(ui->axialViewBtn->isChecked()){
    if(value){            
    //enable boton M-Mode
    ui->mModeBtn->setEnabled(true);

    //enable boton moveLV (move in left ventricle)
    ui->opticalFlowBtn->setEnabled(true);

    //initial values for rotation
    ui->rotXSld->setRange(-90,90);
    ui->rotXSld->setTickInterval(1);
    ui->rotXSld->setValue(0);

    ui->rotYSld->setRange(-90,90);
    ui->rotYSld->setTickInterval(1);
    ui->rotYSld->setValue(0);

    ui->rotZSld->setRange(-90,90);
    ui->rotZSld->setTickInterval(1);
    ui->rotZSld->setValue(0);

    //disable rotation
    ui->rotXSld->setDisabled(true);
    ui->rotYSld->setDisabled(true);
    ui->rotZSld->setDisabled(true);
    //disable slice
    ui->sliceSld->setDisabled(true);

    //read itk volume 4d 
    Image4Type::RegionType inputRegion = 
    vol4.GetPointer()->GetLargestPossibleRegion();
    Image4Type::SizeType vol4size = inputRegion.GetSize();
    Image4Type::IndexType vol4start = inputRegion.GetIndex();

    int nOfSequences = vol4size[3]-1;

    //collapse sequence, 4D->3D
    vol4size[3] = 0;

    //collapse volume, 3D->2D
    std::cout << "Slice: " << ui->sliceSld->value()<< std::endl;
    vol4size[2] = 0;
    vol4start[2] = ui->sliceSld->value();
      
    //create itk 2d image type
    typedef itk::Image<unsigned char, 2> ImageType;
      
    imgSecVTK.resize(nOfSequences+1);
        
    //save 2d vtk images of sequence in a vector
    for (int s=0; s<=nOfSequences; s++) {
      //sequence number
      vol4start[3] = s;

      //desire Region
      Image4Type::RegionType desiredRegion;
      desiredRegion.SetSize(  vol4size  );
      desiredRegion.SetIndex( vol4start );
        
      typedef itk::ExtractImageFilter< Image4Type, ImageType  >  
      ExtractFilterType;
      ExtractFilterType::Pointer filter = ExtractFilterType::New();
      filter->InPlaceOn();
      filter->SetDirectionCollapseToSubmatrix();
      //Set desire region to filter
      filter->SetExtractionRegion( desiredRegion );
      filter->SetInput( vol4.GetPointer() );
      filter->Update();

      //transform image itk to image vtk
      typedef itk::ImageToVTKImageFilter<ImageType> itkVtkConverter;
      itkVtkConverter::Pointer converter = itkVtkConverter::New();
      converter->SetInput(filter->GetOutput());
      converter->Update();

      vtkSmartPointer<vtkImageFlip> volFlipY = 
      vtkSmartPointer<vtkImageFlip>::New();
      volFlipY->SetInput(converter->GetOutput());
      volFlipY->SetFilteredAxis(1);
      volFlipY->Update();
        
      imgSecVTK[s] = vtkSmartPointer<vtkImageData>::New();
      imgSecVTK[s] = volFlipY->GetOutput();
      imgSecVTK[s]->Update();        
    }    
    //display actual slice in axial view
    sliceImageAxial = imgSecVTK[ui->sequenceSld->value()].GetPointer();
    sliceImageAxial->Update();
    viewerAxial->SetInput(sliceImageAxial);
    viewerAxial->Render();

    //updating visual references
    viewerSagital->Render();
    viewerCoronal->Render();      
    }else{
    //disable boton M-Mode
    ui->mModeBtn->setDisabled(true);

    //disable boton moveLV (move in left ventricle)
    ui->opticalFlowBtn->setDisabled(true);

    //enable rotation
    ui->rotXSld->setEnabled(true);
    ui->rotYSld->setEnabled(true);
    ui->rotZSld->setEnabled(true);

    //enable slice
    ui->sliceSld->setEnabled(true);
    }    
  }else{
    ui->fixPlaneBox->setChecked(false);
  }
}

void MainWindow::reslice(int slice)
{
  if(ui->sagitalViewBtn->isChecked()){
    
    //change slice
    double translate = slice - sliceSagital;
    transformSagital->Translate(translate*spacing[0],0,0);
    sliceSagital = slice;

    //change visual reference
    double p0Axial[3] = {sliceSagital*spacing[0],dimensions[1]*spacing[1],0.0};
    double p1Axial[3] = {sliceSagital*spacing[0],0.0,0.0};
  
    sagitalRefInAxialView->SetPoint1(p0Axial);
    sagitalRefInAxialView->SetPoint2(p1Axial);
    sagitalRefInAxialView->Update();
    sagitalRefInAxialViewMapper->Update();

    double p0Coronal[3] = {sliceSagital*spacing[0],dimensions[2]*spacing[2],0.0};
    double p1Coronal[3] = {sliceSagital*spacing[0],0.0,0.0};
  
    sagitalRefInCoronalView->SetPoint1(p0Coronal);
    sagitalRefInCoronalView->SetPoint2(p1Coronal);
    sagitalRefInCoronalView->Update();
    sagitalRefInCoronalViewMapper->Update();

    //displaying new slice
    displaySagital();

  }else if(ui->axialViewBtn->isChecked()){

    //change slice
    double translate = slice - sliceAxial;
    transformAxial->Translate(0,0,translate*spacing[2]);
    sliceAxial = slice;

    //change visual reference
    double p0Sagital[3] = {dimensions[1]*spacing[1],sliceAxial*spacing[2],0.0};
    double p1Sagital[3] = {0.0,sliceAxial*spacing[2],0.0};
  
    axialRefInSagitalView->SetPoint1(p0Sagital);
    axialRefInSagitalView->SetPoint2(p1Sagital);
    axialRefInSagitalView->Update();
    axialRefInSagitalViewMapper->Update();

    double p0Coronal[3] = {dimensions[0]*spacing[0],sliceAxial*spacing[2],0.0};
    double p1Coronal[3] = {0.0,sliceAxial*spacing[2],0.0};
  
    axialRefInCoronalView->SetPoint1(p0Coronal);
    axialRefInCoronalView->SetPoint2(p1Coronal);
    axialRefInCoronalView->Update();
    axialRefInCoronalViewMapper->Update();

    //displaying new slice
    displayAxial();

  }else if(ui->coronalViewBtn->isChecked()){

    //change slice
    double translate = slice - sliceCoronal;
    transformCoronal->Translate(0,translate*spacing[1],0);
    sliceCoronal = slice;

    //change visual reference
    double p0Sagital[3] = {sliceCoronal*spacing[1],dimensions[2]*spacing[2],0.0};
    double p1Sagital[3] = {sliceCoronal*spacing[1],0.0,0.0};
  
    coronalRefInSagitalView->SetPoint1(p0Sagital);
    coronalRefInSagitalView->SetPoint2(p1Sagital);
    coronalRefInSagitalView->Update();
    coronalRefInSagitalViewMapper->Update();

    double p0Axial[3] = {dimensions[0]*spacing[0],sliceCoronal*spacing[1],0.0};
    double p1Axial[3] = {0.0,sliceCoronal*spacing[1],0.0};
  
    coronalRefInAxialView->SetPoint1(p0Axial);
    coronalRefInAxialView->SetPoint2(p1Axial);
    coronalRefInAxialView->Update();
    coronalRefInAxialViewMapper->Update();

    //display new slice
    displayCoronal();
  }
}

void MainWindow::rotateX(int angle)
{
  if(ui->sagitalViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleXSagital;
    transformSagital->Translate(-(sliceSagital*spacing[0]),-(rotCenterSagital[0]),-(rotCenterSagital[1]));
    transformSagital->RotateX(rotate);
    transformSagital->Translate((sliceSagital*spacing[0]),(rotCenterSagital[0]),(rotCenterSagital[1]));
    angleXSagital = angle;

    //display new slice
    displaySagital();

  }else if(ui->axialViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleXAxial;
    transformAxial->Translate(-(rotCenterAxial[0]),-(rotCenterAxial[1]),-(sliceAxial*spacing[2]));
    transformAxial->RotateX(rotate);
    transformAxial->Translate((rotCenterAxial[0]),(rotCenterAxial[1]),(sliceAxial*spacing[2]));
    angleXAxial = angle;

    //display new slice
    displayAxial();

  }else if(ui->coronalViewBtn->isChecked()){

    double rotate = angle - angleXCoronal;
    transformCoronal->Translate(-(rotCenterCoronal[0]),-(sliceCoronal*spacing[1]),-(rotCenterCoronal[1]));
    transformCoronal->RotateX(rotate);
    transformCoronal->Translate((rotCenterCoronal[0]),(sliceCoronal*spacing[1]),(rotCenterCoronal[1]));
    angleXCoronal = angle;

    //display new slice
    displayCoronal();

  }

}

void MainWindow::rotateY(int angle)
{
  if(ui->sagitalViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleYSagital;
    transformSagital->Translate(-(sliceSagital*spacing[0]),-(rotCenterSagital[0]),-(rotCenterSagital[1]));
    transformSagital->RotateY(rotate);
    transformSagital->Translate((sliceSagital*spacing[0]),(rotCenterSagital[0]),(rotCenterSagital[1]));
    angleYSagital = angle;

    //display new slice
    displaySagital();

  }else if(ui->axialViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleYAxial;
    transformAxial->Translate(-(rotCenterAxial[0]),-(rotCenterAxial[1]),-(sliceAxial*spacing[2]));
    transformAxial->RotateY(rotate);
    transformAxial->Translate((rotCenterAxial[0]),(rotCenterAxial[1]),(sliceAxial*spacing[2]));
    angleYAxial = angle;

    //display new slice
    displayAxial();

  }else if(ui->coronalViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleYCoronal;
    transformCoronal->Translate(-(rotCenterCoronal[0]),-(sliceCoronal*spacing[1]),-(rotCenterCoronal[1]));
    transformCoronal->RotateY(rotate);
    transformCoronal->Translate((rotCenterCoronal[0]),(sliceCoronal*spacing[1]),(rotCenterCoronal[1]));
    angleYCoronal = angle;

    //display new slice
    displayCoronal();

  }

}

void MainWindow::rotateZ(int angle)
{
  if(ui->sagitalViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleZSagital;
    transformSagital->Translate(-(sliceSagital*spacing[0]),-(rotCenterSagital[0]),-(rotCenterSagital[1]));
    transformSagital->RotateZ(rotate);
    transformSagital->Translate((sliceSagital*spacing[0]),(rotCenterSagital[0]),(rotCenterSagital[1]));
    angleZSagital = angle;

    //display new slice
    displaySagital();

  }else if(ui->axialViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleZAxial;
    transformAxial->Translate(-(rotCenterAxial[0]),-(rotCenterAxial[1]),-(sliceAxial*spacing[2]));
    transformAxial->RotateZ(rotate);
    transformAxial->Translate((rotCenterAxial[0]),(rotCenterAxial[1]),(sliceAxial*spacing[2]));
    angleZAxial = angle;

    //display new slice
    displayAxial();

  }else if(ui->coronalViewBtn->isChecked()){

    //rotate slice
    double rotate = angle - angleZCoronal;
    transformCoronal->Translate(-(rotCenterCoronal[0]),-(sliceCoronal*spacing[1]),-(rotCenterCoronal[1]));
    transformCoronal->RotateZ(rotate);
    transformCoronal->Translate((rotCenterCoronal[0]),(sliceCoronal*spacing[1]),(rotCenterCoronal[1]));
    angleZCoronal = angle;

    //display new slice
    displayCoronal();

  }
}

void MainWindow::sagitalBtnClicked(bool value)
{    
  //check button
  ui->sagitalViewBtn->setChecked(true);

  addActorsSagitalView();

  if(value){

    //change the view when only one view is active 
    if(ui->displayOneBtn->isChecked()){
      sagitalWidget->show();
      axialWidget->hide();
      coronalWidget->hide();
      volWidget->hide();
    }
    
    if(ui->axialViewBtn->isChecked()){

      //uncheck button
      ui->axialViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[2]-1)>=(dimensions[0]-1)){
        ui->sliceSld->setValue(sliceSagital);
        ui->sliceSld->setRange(0,dimensions[0]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[0]-1);
        ui->sliceSld->setValue(sliceSagital);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleAxial->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleAxial->clearTracer();
        viewerAxial->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleAxial);
        viewerAxial->Render();
      }

    }else if(ui->coronalViewBtn->isChecked()){

      //uncheck button
      ui->coronalViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[1]-1)>=(dimensions[0]-1)){
        ui->sliceSld->setValue(sliceSagital);
        ui->sliceSld->setRange(0,dimensions[0]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[0]-1);
        ui->sliceSld->setValue(sliceSagital);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleCoronal->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleCoronal->clearTracer();
        viewerCoronal->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleCoronal);
        viewerCoronal->Render();
      }

    }else if(ui->volViewBtn->isChecked()){
      
      //uncheck button
      ui->volViewBtn->setChecked(false);
    
    }

    //change rotation sliders to current position
    ui->rotXSld->setValue(angleXSagital);
    ui->rotYSld->setValue(angleYSagital);
    ui->rotZSld->setValue(angleZSagital);
      
  }
}

void MainWindow::axialBtnClicked(bool value)
{
  //check button
  ui->axialViewBtn->setChecked(true);
  
  addActorsAxialView();

  if(value){

    //change the view when only one view is active 
    if(ui->displayOneBtn->isChecked()){
      sagitalWidget->hide();
      axialWidget->show();
      coronalWidget->hide();
      volWidget->hide();
    }

    //remove observer
    imageStyleAxial->RemoveAllObservers();

    if(ui->sagitalViewBtn->isChecked()){

      //uncheck button
      ui->sagitalViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[0]-1)>=(dimensions[2]-1)){
        ui->sliceSld->setValue(sliceAxial);
        ui->sliceSld->setRange(0,dimensions[2]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[2]-1);
        ui->sliceSld->setValue(sliceAxial);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleSagital->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleSagital->clearTracer();
        viewerSagital->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleSagital);
        viewerSagital->Render();
      }

    }else if(ui->coronalViewBtn->isChecked()){

      //uncheck button
      ui->coronalViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[1]-1)>=(dimensions[2]-1)){
        ui->sliceSld->setValue(sliceAxial);
        ui->sliceSld->setRange(0,dimensions[2]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[2]-1);
        ui->sliceSld->setValue(sliceAxial);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleCoronal->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleCoronal->clearTracer();
        viewerCoronal->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleCoronal);
        viewerCoronal->Render();
      }

    }else if(ui->volViewBtn->isChecked()){
      
      //uncheck button
      ui->volViewBtn->setChecked(false);
    
    }

    //change rotation sliders to current position
    ui->rotXSld->setValue(angleXAxial);
    ui->rotYSld->setValue(angleYAxial);
    ui->rotZSld->setValue(angleZAxial);
      
  }
}

void MainWindow::coronalBtnClicked(bool value)
{
  //check button
  ui->coronalViewBtn->setChecked(true);

  addActorsCoronalView();

  if(value){

    //change the view when only one view is active 
    if(ui->displayOneBtn->isChecked()){
      sagitalWidget->hide();
      axialWidget->hide();
      coronalWidget->show();
      volWidget->hide();
    }

    //remove observer
    imageStyleCoronal->RemoveAllObservers();

    if(ui->sagitalViewBtn->isChecked()){

      //uncheck button
      ui->sagitalViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[0]-1)>=(dimensions[1]-1)){
        ui->sliceSld->setValue(sliceCoronal);
        ui->sliceSld->setRange(0,dimensions[1]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[1]-1);
        ui->sliceSld->setValue(sliceCoronal);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleSagital->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleSagital->clearTracer();
        viewerSagital->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleSagital);
        viewerSagital->Render();
      }

    }else if(ui->axialViewBtn->isChecked()){

      //uncheck button
      ui->axialViewBtn->setChecked(false);

      //change reslice slider to current position
      if((dimensions[2]-1)>=(dimensions[0]-1)){
        ui->sliceSld->setValue(sliceCoronal);
        ui->sliceSld->setRange(0,dimensions[1]-1);
      }else{
        ui->sliceSld->setRange(0,dimensions[1]-1);
        ui->sliceSld->setValue(sliceCoronal);
      }

      //uncheck buttons and return view to normal
      if(ui->rotCenterBtn->isChecked()){
        ui->rotCenterBtn->setChecked(false);
        imageStyleAxial->RemoveAllObservers();
      }

      if(ui->segmentBtn->isChecked()){
        ui->segmentBtn->setChecked(false);
        segmentationStyleAxial->clearTracer();
        viewerAxial->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleAxial);
        viewerAxial->Render();
      }

    }else if(ui->volViewBtn->isChecked()){
      
      //uncheck button
      ui->volViewBtn->setChecked(false);
    
    }

    //change rotation sliders to current position
    ui->rotXSld->setValue(angleXCoronal);
    ui->rotYSld->setValue(angleYCoronal);
    ui->rotZSld->setValue(angleZCoronal);
      
  }
}

void MainWindow::volBtnClicked(bool value)
{
  //check button
  ui->volViewBtn->setChecked(true);
  
  if(value){

    //uncheck buttons
    ui->rotCenterBtn->setChecked(false);

    //change the view when only one view is active 
    if(ui->displayOneBtn->isChecked()){
      sagitalWidget->hide();
      axialWidget->hide();
      coronalWidget->hide();
      volWidget->show();
    }

    //uncheck buttons
    ui->sagitalViewBtn->setChecked(false);
    ui->axialViewBtn->setChecked(false);
    ui->coronalViewBtn->setChecked(false);
  }
}

void MainWindow::allBtnClicked(bool value)
{
  //check and uncheck view buttons
  ui->displayAllBtn->setChecked(true);
  ui->displayOneBtn->setChecked(false);

  //show all views
  if(value){
    sagitalWidget->show();
    axialWidget->show();
    coronalWidget->show();
    volWidget->show();
  }
}

void MainWindow::oneBtnClicked(bool value)
{
  //check and uncheck view buttons
  ui->displayOneBtn->setChecked(true);
  ui->displayAllBtn->setChecked(false);

  if(value){

    if(ui->sagitalViewBtn->isChecked()){

      //showing selected view
      sagitalWidget->show();
      axialWidget->hide();
      coronalWidget->hide();
      volWidget->hide();

    }else if(ui->axialViewBtn->isChecked()){

      //showing selected view
      sagitalWidget->hide();
      axialWidget->show();
      coronalWidget->hide();
      volWidget->hide();

    }else if(ui->coronalViewBtn->isChecked()){

      //showing selected view
      sagitalWidget->hide();
      axialWidget->hide();
      coronalWidget->show();
      volWidget->hide();

    }else if(ui->volViewBtn->isChecked()){

      //showing selected view
      sagitalWidget->hide();
      axialWidget->hide();
      coronalWidget->hide();
      volWidget->show();

    }
  }
}

void MainWindow::rotCenterBtnClicked(bool value)
{
  if(value){
    
    //initialize picker objects
    vtkSmartPointer<vtkPropPicker> propPicker = vtkSmartPointer<vtkPropPicker>::New();
    vtkSmartPointer<QVTKImageWidgetCommand<MainWindow> > callback = vtkSmartPointer<QVTKImageWidgetCommand<MainWindow> >::New();

    callback->SetImageWidget(this);
    propPicker->PickFromListOn();
    callback->SetPicker(propPicker);
    
    if(ui->sagitalViewBtn->isChecked()){

      std::cout<<std::endl<<"Change rotation center in sagital view"<<std::endl;

      //add observer to selecter view
      propPicker->AddPickList(imageActorSagital);
      callback->SetImageViewer(viewerSagital);
      imageStyleSagital->AddObserver(vtkCommand::LeftButtonPressEvent, callback);

    }else if(ui->axialViewBtn->isChecked()){

      std::cout<<std::endl<<"Change rotation center in axial view"<<std::endl;

      //add observer to selecter view
      propPicker->AddPickList(imageActorAxial);
      callback->SetImageViewer(viewerAxial);
      imageStyleAxial->AddObserver(vtkCommand::LeftButtonPressEvent, callback);

    }else if(ui->coronalViewBtn->isChecked()){

      std::cout<<std::endl<<"Change rotation center in coronal view"<<std::endl;

      //add observer to selecter view
      propPicker->AddPickList(imageActorCoronal);
      callback->SetImageViewer(viewerCoronal);
      imageStyleCoronal->AddObserver(vtkCommand::LeftButtonPressEvent, callback);

    }
  }else{
  
    //remove observers from all views
    imageStyleSagital->RemoveAllObservers();
    imageStyleAxial->RemoveAllObservers();
    imageStyleCoronal->RemoveAllObservers();

  }
}

void MainWindow::segmentBtnClicked(bool value)
{  
  if(value){
  
    if(ui->sagitalViewBtn->isChecked()){
      
      //initialize segmentation interactor style in selected view
      segmentationStyleSagital = vtkSmartPointer< vtkTracerInteractorStyle<MainWindow> >::New();
      segmentationStyleSagital->setCallerWidget(this);
      viewerSagital->GetRenderWindow()->GetInteractor()->SetInteractorStyle(segmentationStyleSagital);
      segmentationStyleSagital->initTracer(imageActorSagital);
      segmentationStyleSagital->setAutoCloseOn();

      viewerSagital->GetRenderer()->RemoveActor(sagitalCenterRefActor);

    }else if(ui->axialViewBtn->isChecked()){
      
      //initialize segmentation interactor style in selected view
      segmentationStyleAxial = vtkSmartPointer< vtkTracerInteractorStyle<MainWindow> >::New();
      segmentationStyleAxial->setCallerWidget(this);
      viewerAxial->GetRenderWindow()->GetInteractor()->SetInteractorStyle(segmentationStyleAxial);
      segmentationStyleAxial->initTracer(imageActorAxial);
      segmentationStyleAxial->setAutoCloseOn();

      viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);

    }else if(ui->coronalViewBtn->isChecked()){
      
      //initialize segmentation interactor style in selected view
      segmentationStyleCoronal = vtkSmartPointer< vtkTracerInteractorStyle<MainWindow> >::New();
      segmentationStyleCoronal->setCallerWidget(this);
      viewerCoronal->GetRenderWindow()->GetInteractor()->SetInteractorStyle(segmentationStyleCoronal);
      segmentationStyleCoronal->initTracer(imageActorCoronal);
      segmentationStyleCoronal->setAutoCloseOn();

      viewerCoronal->GetRenderer()->RemoveActor(coronalCenterRefActor);

    }

  }else{

    if(ui->sagitalViewBtn->isChecked()){

      //removing segmentation interactor style from selected view 
      segmentationStyleSagital->clearTracer();
      viewerSagital->GetRenderer()->AddActor(sagitalCenterRefActor);
      viewerSagital->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleSagital);

    }else if(ui->axialViewBtn->isChecked()){

      //removing segmentation interactor style from selected view 
      segmentationStyleAxial->clearTracer();
      viewerAxial->GetRenderer()->AddActor(axialCenterRefActor);
      viewerAxial->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleAxial);

    }else if(ui->coronalViewBtn->isChecked()){

      //removing segmentation interactor style from selected view 
      segmentationStyleCoronal->clearTracer();
      viewerCoronal->GetRenderer()->AddActor(coronalCenterRefActor);
      viewerCoronal->GetRenderWindow()->GetInteractor()->SetInteractorStyle(imageStyleCoronal);

    }

  }
}

void MainWindow::opticalFlowBtnClicked(bool value){
  if( value ){
    if(ui->axialViewBtn->isChecked()){ 
	  //opticalFlowHermite(vector of VTK images 2D,scale,iterations)
      opticalFlowHermite(imgSecVTK,1,1);

        //no checked optical flow boton 
      ui->opticalFlowBtn->setChecked(false);

      //Window OpticalFlow
      OpticalFlowWidget *widget = new OpticalFlowWidget;
      widget->setWindowTitle("Optical Flow");
      widget->setOpticalFlowData(imgSecVTK,OFduITK,OFdvITK);
      if(mModeFlag){
        widget->setMmodeData(mMode,periodMmode,ciclesMmode);
      }
      widget->show();  
      
    }else{
      //no checked optical flow boton 
      ui->opticalFlowBtn->setChecked(false);
    }
      
  }
}

void MainWindow::opticalFlowHermite(std::vector< vtkSmartPointer<vtkImageData> > imgVTK,int scale, int iterations)
{
  typedef itk::Image<unsigned char, 2> InputImageType;
  typedef itk::Image<double, 2> OutputImageType;
  typedef itk::HermiteOpticalFlow<InputImageType,OutputImageType> OpticalFlowType;

  OFduITK.resize(imgVTK.size());
  OFdvITK.resize(imgVTK.size());

  std::cout<<std::endl;
  std::cout<<"Optical Flow with "<<scale<<" scales and "<<iterations<<
	  " iterations"<<std::endl;

  //OF in secuence
  for(int numSec = 0; numSec < imgVTK.size()-1; numSec++){
    std::cout<<"image"<<numSec<<std::endl;
    //Transform VTK image to ITK image; itkVTKImageToImageFilter only accepts single channel images
    itk::VTKImageToImageFilter<InputImageType> ::Pointer vtkImageToImageFilter0 = 
		itk::VTKImageToImageFilter<InputImageType> ::New();
      vtkImageToImageFilter0->SetInput(imgVTK[numSec]);
      vtkImageToImageFilter0->Update();

    itk::VTKImageToImageFilter<InputImageType> ::Pointer vtkImageToImageFilter1 = 
		itk::VTKImageToImageFilter<InputImageType> ::New();
      vtkImageToImageFilter1->SetInput(imgVTK[numSec+1]);
      vtkImageToImageFilter1->Update();

    //Calcule Optical Flow betwen 2 ITK images    
    OpticalFlowType::Pointer opticalFlow = OpticalFlowType::New();
    opticalFlow->SetInputImage0(vtkImageToImageFilter0->GetOutput());
    opticalFlow->SetInputImage1(vtkImageToImageFilter1->GetOutput());
    opticalFlow->SetScale(scale);
    opticalFlow->SetIterations(iterations);
    opticalFlow->Update();

    //Save OF in vectors     
    OFduITK[numSec] = OutputImageType::New();
    OFduITK[numSec] = opticalFlow->GetDu();
    
    OFdvITK[numSec] = OutputImageType::New();
    OFdvITK[numSec] = opticalFlow->GetDv();      
  }

  //OF bettewn last and first images
  //Transform last and first VTK image to ITK image
  std::cout<<"image"<<imgVTK.size()-1<<std::endl;
  itk::VTKImageToImageFilter<InputImageType> ::Pointer vtkImageToImageFilter0 = 
	  itk::VTKImageToImageFilter<InputImageType> ::New();
    vtkImageToImageFilter0->SetInput(imgVTK[imgVTK.size()-1]);
    vtkImageToImageFilter0->Update();

  itk::VTKImageToImageFilter<InputImageType> ::Pointer vtkImageToImageFilter1 = 
	  itk::VTKImageToImageFilter<InputImageType> ::New();
    vtkImageToImageFilter1->SetInput(imgVTK[0]);
    vtkImageToImageFilter1->Update();

  //Calcule Optical Flow betwen last and first ITK images  
  OpticalFlowType::Pointer opticalFlow = OpticalFlowType::New();
  opticalFlow->SetInputImage0(vtkImageToImageFilter0->GetOutput());
  opticalFlow->SetInputImage1(vtkImageToImageFilter1->GetOutput());
  opticalFlow->SetScale(scale);
  opticalFlow->SetIterations(iterations);
  opticalFlow->Update();

  //Save OF in vectors 
  OFduITK[imgVTK.size()-1] = OutputImageType::New();
  OFduITK[imgVTK.size()-1] = opticalFlow->GetDu();

  OFdvITK[imgVTK.size()-1] = OutputImageType::New();
  OFdvITK[imgVTK.size()-1] = opticalFlow->GetDv();

}

void MainWindow::resetSlice()
{
  if(ui->sagitalViewBtn->isChecked()){
    
    std::cout<<"Reset Sagital View to center slice"<<std::endl;

    //update data and actors of current view
    removeActorsSagitalView();
    configSagitalView();
    addActorsSagitalView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);;
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[0]);

  }else if(ui->axialViewBtn->isChecked()){
      
    std::cout<<"Reset Axial View to center slice"<<std::endl;

    //update data and actors of current view
    removeActorsAxialView();
    configAxialView();
    addActorsAxialView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[2]);

  }else if(ui->coronalViewBtn->isChecked()){

    std::cout<<"Reset Coronal View to center slice"<<std::endl;

    //update data and actors of current view
    removeActorsCoronalView();
    configCoronalView();
    addActorsCoronalView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[1]);

  }
}

void MainWindow::resetAll()
{
  std::cout<<"Reset All Views to center slice"<<std::endl;

  //update data from all views
  removeActorsSagitalView();
  configSagitalView();
  
  removeActorsAxialView();
  configAxialView();

  removeActorsCoronalView();
  configCoronalView();

  if(ui->sagitalViewBtn->isChecked()){
        
    //add actors to current view
    addActorsSagitalView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);;
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[0]);

    displayAxial();
    displayCoronal();

  }else if(ui->axialViewBtn->isChecked()){
    
    //add actors to current view
    addActorsAxialView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[2]);

    displaySagital();
    displayCoronal();

  }else if(ui->coronalViewBtn->isChecked()){

    //add actors to current view
    addActorsCoronalView();

    //setting initial values for sliders
    ui->rotXSld->setValue(0);
    ui->rotYSld->setValue(0);
    ui->rotZSld->setValue(0);
    ui->sliceSld->setValue(centerSlice[1]);

    displayAxial();
    displaySagital();

  }
}

void MainWindow::saveSeg()
{
  segmented = true;
  ui->saveSegBtn->setEnabled(true);

  QString saveDirectory = QFileDialog::getSaveFileName(
                this, tr("Choose File to Save Mask Volume"), QDir::currentPath());

  QString saveMhdDirectory = saveDirectory;
  QString saveRawDirectory = saveDirectory;

    QString mhdFilename = ".mhd";
  QString rawFilename = ".raw";

    QString qtSaveMhdFile = saveMhdDirectory.append(mhdFilename);
  QString qtSaveRawFile = saveRawDirectory.append(rawFilename);

  std::string str1 = std::string(qtSaveMhdFile.toAscii().data());
  const char * saveMhdFile = str1.c_str();

  std::string str2 = std::string(qtSaveRawFile.toAscii().data());
  const char * saveRawFile = str2.c_str();

  std::cout<<"Saving Mask Volume in files:"<<std::endl<<std::endl;
  std::cout<<saveMhdFile<<std::endl<<std::endl;
  std::cout<<saveRawFile<<std::endl;

  vtkSmartPointer<vtkMetaImageWriter> writer = vtkSmartPointer<vtkMetaImageWriter>::New();
  writer->SetFileName(saveMhdFile);
  writer->SetRAWFileName(saveRawFile);
  //writer->SetInput(blend->GetOutput());

  try{
  writer->Write();
  }catch(std::exception& e){
    std::cout<<e.what()<<std::endl;
  }
}

void MainWindow::changeDisplayedSeq(int seq)
{
  if(ui->fixPlaneBox->isChecked()){
    ui->sliceSld->update();

    //display only axial view
    sliceImageAxial = imgSecVTK[seq];
    sliceImageAxial->Update();
    viewerAxial->SetInput(sliceImageAxial);
    viewerAxial->Render();

    //updating visual references
    viewerSagital->Render();
    viewerCoronal->Render();
  }else{
    volumeData = volumesVTK[seq];

    ui->sliceSld->update();

    //displaying volume
    setRenderingData();
    displayVol();

    //configureation of 2D views
    configSagitalView();
    configAxialView();
    configCoronalView();

    addActorsAxialView();

    //displaying 2D views
    displaySagital();
    displayCoronal();

    //displaying 2D views
    reslice(ui->sliceSld->value());
  }
}

void MainWindow::mModeBtnClicked(bool value){
  if(value){
  
    if(ui->axialViewBtn->isChecked()){      
      //initialize segmentation interactor style in selected view
      mModeStyle = 
      vtkSmartPointer< vtkTracerInteractorStyle<MainWindow> >::New();
      mModeStyle->setCallerWidget(this);
      viewerAxial->GetRenderWindow()->GetInteractor()->
      SetInteractorStyle(mModeStyle);
      mModeStyle->initTracer(imageActorAxial);
      mModeStyle->setAutoCloseOff();

      viewerAxial->GetRenderer()->RemoveActor(axialCenterRefActor);
    }else{
      ui->mModeBtn->setChecked(false);
    }
  }else{
    if(ui->axialViewBtn->isChecked()){
      //removing m-mode interactor style from selected view 
      mModeStyle->clearTracer();
      viewerAxial->GetRenderer()->AddActor(axialCenterRefActor);
      viewerAxial->GetRenderWindow()->GetInteractor()->
      SetInteractorStyle(imageStyleAxial);
    }
  }
}
