/*
 * File:   M Mode
 * Author: Fabiola Pacheco
 * 
 * Created on 01/30/2017
 * Last modification on 09/29/2017
 */
#include "mmodewidget.h"
#include "ui_mmodewidget.h"

#include <QVBoxLayout>

#include <vtkImageShiftScale.h>
#include <vtkPropPicker.h>

#include "QVTKImageWidgetCommand.h"


MmodeWidget::MmodeWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MmodeWidget)
{
    ui->setupUi(this);

  //initializing M-Mode display widget
  imageWidget = new QVTKWidget;
  QVBoxLayout *imageLayout = new QVBoxLayout;
  imageLayout->setContentsMargins(0, 0, 0, 0);
  imageLayout->setSpacing(0);
  imageLayout->QLayout::addWidget(imageWidget);
  ui->imageWidget->setLayout(imageLayout);
}

MmodeWidget::~MmodeWidget()
{
    delete ui;
}

void MmodeWidget::setImageData(vtkSmartPointer<vtkImageData> image) 
{
  mMode = image;
  mMode2 = image;

  //initializing 2D view
  imageStyleImage = vtkSmartPointer<vtkInteractorStyleImage>::New();
  viewerImage = vtkSmartPointer<vtkImageViewer2>::New();
  imageActor = vtkSmartPointer<vtkImageActor>::New();
  
  //initialize visual reference
  imageRefLineViewB = vtkSmartPointer<vtkLineSource>::New();
  imageRefLineViewE = vtkSmartPointer<vtkLineSource>::New();
  imageRefLineViewMapperB = vtkSmartPointer<vtkPolyDataMapper>::New();
  imageRefLineViewMapperE = vtkSmartPointer<vtkPolyDataMapper>::New();
  imageRefLineViewActorB = vtkSmartPointer<vtkActor>::New();
  imageRefLineViewActorE = vtkSmartPointer<vtkActor>::New();

  imageRefLineViewActorB->GetProperty()->SetLineWidth(1);
  imageRefLineViewActorE->GetProperty()->SetLineWidth(1);
  imageRefLineViewActorB->GetProperty()->SetColor(237,0,0);
  imageRefLineViewActorE->GetProperty()->SetColor(237,0,0);
  imageRefLineViewMapperB->SetInputConnection
	  (imageRefLineViewB->GetOutputPort());
  imageRefLineViewMapperE->SetInputConnection
	  (imageRefLineViewE->GetOutputPort());
  imageRefLineViewActorB->SetMapper(imageRefLineViewMapperB);
  imageRefLineViewActorE->SetMapper(imageRefLineViewMapperE);

  //initialize cicles
  cicles = 1;

  //set image to viewer
  viewerImage->SetInput(image);

  imageWidget->SetRenderWindow(viewerImage->GetRenderWindow());
  imageActor = viewerImage->GetImageActor();
  //imageActor->InterpolateOff();
  //imageActor->InterpolateOn();//default
  viewerImage->GetRenderWindow()->GetInteractor()->
	  SetInteractorStyle(imageStyleImage);
  viewerImage->Render();  

  setCicles(1);

  //initialize period flag
  periodFlag = false;
}

void MmodeWidget::invertIntensityBoxClicked(bool value){
  //invert intensity filter
  vtkSmartPointer<vtkImageShiftScale> inverter = 
	  vtkSmartPointer<vtkImageShiftScale>::New();
  inverter->SetOutputScalarTypeToUnsignedChar();
  inverter->SetShift(-255.0);
  inverter->SetScale(-1.0);

  if(value){
    if(ui->invertImageBox->isChecked()){
      //invert intensity of inverted image
      inverter->SetInput(mModeFlipY);
      inverter->Update();
    }else{
      //invert intensity
      inverter->SetInput(mMode2);
      inverter->Update();
    }
    //display image with invert intensity
    viewerImage->SetInput(inverter->GetOutput());
  }else{
    if(ui->invertImageBox->isChecked()){
      //display inverted image
      viewerImage->SetInput(mModeFlipY);
    }else{
      //display original image
      viewerImage->SetInput(mMode2);
    }
  }
  viewerImage->Render();  
}

void MmodeWidget::invertImageBoxClicked(bool value){  
  
  if(value){
    if(ui->invertColorBox->isChecked()){
      //invert intensity of inverted image
      vtkSmartPointer<vtkImageShiftScale> inverter = 
		  vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mModeFlipY);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      //display image with invert intensity
      viewerImage->SetInput( inverter->GetOutput() );
    }else{
      //display inverted image
      viewerImage->SetInput( mModeFlipY );
    }    
  }else{
    if(ui->invertColorBox->isChecked()){
      //invert image
      vtkSmartPointer<vtkImageShiftScale> inverter = 
		  vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mMode2);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      //display image with invert intensity
      viewerImage->SetInput(inverter->GetOutput());
    }else{
      //display original image
      viewerImage->SetInput(mMode2);
    }    
  }
  viewerImage->Render();  
}

void MmodeWidget::selectPeriodBtnClicked(bool value){
  //initialize picker objects
  vtkSmartPointer<vtkPropPicker> propPicker = 
	  vtkSmartPointer<vtkPropPicker>::New();
  vtkSmartPointer<QVTKImageWidgetCommand<MmodeWidget> > callback = 
	  vtkSmartPointer<QVTKImageWidgetCommand<MmodeWidget> >::New();
  
  callback->SetImageWidget(this);
  propPicker->PickFromListOn();
  callback->SetPicker(propPicker);

  if(value){    
    //add observer to selecter view
    propPicker->AddPickList(imageActor);
    callback->SetImageViewer(viewerImage);
    imageStyleImage->AddObserver(vtkCommand::LeftButtonPressEvent, 
		                         callback);

    //initialize flag
    periodFlag = true;

    //remove last references
    viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorB);
    viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorE);

    //disable box for repeat m-mode
    ui->repeatBox->setDisabled(true);  
  }else{
    //remove picker objects
    imageStyleImage->RemoveAllObservers();  
    
    //remove last references
    viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorB);
    viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorE);

    //enable box for repeat m-mode
    ui->repeatBox->setEnabled(true);    
  }
  viewerImage->Render();  
}

void MmodeWidget::setNewClickedPosition(double x, double y)
{
   double *spacing = mMode2->GetSpacing();

   //select period values
   if(x!=0){ 
     if(periodFlag){
       period[0] = vtkMath::Floor((x/spacing[0]) + 0.5);

      //remove last references
      viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorB);
      viewerImage->GetRenderer()->RemoveActor(imageRefLineViewActorE);

      //position of new reference
      imageRefLineViewB->SetPoint1(x,0,0);
      imageRefLineViewB->SetPoint2
		  (x,(mMode2->GetDimensions()[1]-1)*spacing[1],0);
      imageRefLineViewB->Update();

      imageRefLineViewMapperB->Update();  
    
      viewerImage->GetRenderer()->AddActor(imageRefLineViewActorB);
      
      periodFlag = false;
     }else{//Ending of period
      period[1] = vtkMath::Floor((x/spacing[0]) + 0.5);

      //position of new reference
      imageRefLineViewE->SetPoint1(x,0,0);
      imageRefLineViewE->SetPoint2
		  (x,(mMode2->GetDimensions()[1]-1)*spacing[1],0);
      imageRefLineViewE->Update();
  
      imageRefLineViewMapperE->Update();  

      viewerImage->GetRenderer()->AddActor(imageRefLineViewActorE);
      periodFlag = true;

      //Sort from lowest to highest period 
      if(period[0]>period[1]){
        int p = period[0];
        period[0] = period[1];
        period[1] = p;
      }
      //return period to MainWindow
      MainWindow->setPeriod(period[0],period[1],cicles);
      
      viewerImage->RemoveAllObservers();
     }
   }   
   viewerImage->Render();      
}

void MmodeWidget::setCicles(int num){
    cicles = num;

  //resize M-mode2 for save "num" cicles
  mMode2 = vtkSmartPointer<vtkImageData>::New();
  mMode2->SetOrigin( mMode->GetOrigin() );
  mMode2->SetSpacing( mMode->GetSpacing() );
  mMode2->SetDimensions
	  ( mMode->GetDimensions()[0]*num , mMode->GetDimensions()[1],1 );
  mMode2->SetScalarTypeToUnsignedChar();
  mMode2->SetNumberOfScalarComponents(1);
  mMode2->Update();

  mModeFlipY = vtkSmartPointer<vtkImageData>::New();
  mModeFlipY->SetOrigin( mMode->GetOrigin() );
  mModeFlipY->SetSpacing( mMode->GetSpacing() );
  mModeFlipY->SetDimensions
	  ( mMode->GetDimensions()[0]*num , mMode->GetDimensions()[1],1 );  
  mModeFlipY->SetScalarTypeToUnsignedChar();
  mModeFlipY->SetNumberOfScalarComponents(1);
  mModeFlipY->Update();

  //fill M-Mode2 and  flip M-Mode2
  for(int x = 0; x < mMode->GetDimensions()[0]; x++){
    for(int y = 0; y < mMode->GetDimensions()[1]; y++){
      for(int n = 0; n < num; n++){
        unsigned char * valueMmode = static_cast<unsigned char *> 
			( mMode2->GetScalarPointer((n*(mMode->GetDimensions()[0]))+
			  x,y,0) );
        unsigned char * valueFlipMmode = static_cast<unsigned char *> 
			( mModeFlipY->GetScalarPointer((n*(mMode->GetDimensions()[0]))+
			  x,mMode->GetDimensions()[1]-1-y,0) );
        valueMmode[0] = static_cast<unsigned char *> 
			( mMode->GetScalarPointer(x,y,0) )[0];
        valueFlipMmode[0] = static_cast<unsigned char *> 
			( mMode->GetScalarPointer(x,y,0) )[0];
      }
    }
  }  

  mMode2->Update();
  mModeFlipY->Update();

  if(ui->invertImageBox->isChecked()){
    if(ui->invertColorBox->isChecked()){
      //invert intensity
      vtkSmartPointer<vtkImageShiftScale> inverter = 
		  vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mModeFlipY);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      //display
      viewerImage->SetInput(inverter->GetOutput());
    }else{
      //display
      viewerImage->SetInput(mModeFlipY);
    }
  }else{
    if(ui->invertColorBox->isChecked()){
      //invert intensity
      vtkSmartPointer<vtkImageShiftScale> inverter = 
		  vtkSmartPointer<vtkImageShiftScale>::New();
      inverter->SetInput(mMode2);
      inverter->SetOutputScalarTypeToUnsignedChar();
      inverter->SetShift(-255.0);
      inverter->SetScale(-1.0);
      inverter->Update();

      //display
      viewerImage->SetInput(inverter->GetOutput());

    }else{
      //display
      viewerImage->SetInput(mMode2);
    }
  }

  viewerImage->GetRenderer()->ResetCamera();//center image
  viewerImage->Render();      
}

void MmodeWidget::setMainWindow(MainWindowType * mainWindow){
    this->MainWindow = mainWindow;
}
