/*
 * File:   M Mode
 * Author: Fabiola Pacheco
 * 
 * Created on 01/30/2017
 * Last modification on 09/29/2017
 */
#ifndef MMODEWIDGET_H
#define MMODEWIDGET_H

#include <QWidget>
#include <QVTKWidget.h>
#include "mainwindow.h"

#include <vtkRenderWindow.h>
#include <vtkLineSource.h>

namespace Ui {
class MmodeWidget;
}

class MmodeWidget : public QWidget
{
    Q_OBJECT

public:
    explicit MmodeWidget(QWidget *parent = 0);
    ~MmodeWidget();

  typedef class MainWindow MainWindowType ;

  /**
  Set the image to widget
  **/
  void MmodeWidget::setImageData(vtkSmartPointer<vtkImageData>); 
  
  void setNewClickedPosition(double,double);

  void setMainWindow(MainWindowType * mainWindow);

private:
    Ui::MmodeWidget *ui;

  /*
  MainWindow object
  */
  MainWindowType * MainWindow;

  /**
  Widget that display the M-Mode
  **/
  QVTKWidget *imageWidget;

  /**
  Image Viewer that render the image
  **/
  vtkSmartPointer<vtkImageViewer2> viewerImage;

  /**
  ImageActor of the image
  **/
  vtkSmartPointer<vtkImageActor> imageActor;

  /**
  Interactor style of the image viewer, it has the basic interaction with a 2D image
  **/
  vtkSmartPointer<vtkInteractorStyleImage> imageStyleImage;

  /**
  Begining and Ending of Periods  references in view
  **/
  vtkSmartPointer<vtkLineSource> imageRefLineViewB;
  vtkSmartPointer<vtkLineSource> imageRefLineViewE;

  /**
  Begining and Ending of Periods  references in view actor
  **/
  vtkSmartPointer<vtkActor> imageRefLineViewActorB;
  vtkSmartPointer<vtkActor> imageRefLineViewActorE;

  /**
  Begining and Ending of Periods  references in view mapper
  **/
  vtkSmartPointer<vtkPolyDataMapper> imageRefLineViewMapperB;
  vtkSmartPointer<vtkPolyDataMapper> imageRefLineViewMapperE;

  /**
  Period Flag
  **/
  bool periodFlag;

  /**
  period[0]= first number chosen of the secuence image
  period[1]= last number chosen of the secuence image
  **/
  int period[2];

  /**
  M-Mode image
  **/
  vtkSmartPointer<vtkImageData> mMode;

  /**
  repeat M-Mode image
  **/
  vtkSmartPointer<vtkImageData> mMode2;

  /**
  flip repeat M-Mode image
  **/
  vtkSmartPointer<vtkImageData> mModeFlipY;

  /**
  times of M-Mode repeated
  **/
  int cicles;

private slots:
  /**
    Set inversion of intensity of image
    **/
    void invertIntensityBoxClicked(bool);

  /**
    Set inversion of image
    **/
    void invertImageBoxClicked(bool);

  /**
    Set beginning and ending of a period
    **/
  void selectPeriodBtnClicked(bool);

  /**
    Set how many times is M-Mode repeated
    **/
  void setCicles(int);

};

#endif // MMODEWIDGET_H
